# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit test template for ONTAP Ansible module '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_igroup_initiator \
    import NetAppOntapIgroupInitiator as initiator  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, data=None):
        ''' save arguments '''
        self.kind = kind
        self.data = data
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.kind == 'initiator':
            xml = self.build_igroup_initiator()
        self.xml_out = xml
        return xml

    @staticmethod
    def build_igroup_initiator():
        ''' build xml data for initiator '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'num-records': 1,
            'attributes-list': {
                'initiator-group-info': {
                    'initiators': [
                        {'initiator-info': {
                            'initiator-name': 'init1'
                        }},
                        {'initiator-info': {
                            'initiator-name': 'init2'
                        }}
                    ]
                }
            }
        }
        xml.translate_struct(attributes)
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.server = MockONTAPConnection()

    def mock_args(self):
        return {
            'vserver': 'vserver',
            'name': 'init1',
            'initiator_group': 'test',
            'hostname': 'hostname',
            'username': 'username',
            'password': 'password'
        }

    def get_initiator_mock_object(self, kind=None):
        """
        Helper method to return an na_ontap_initiator object
        :param kind: passes this param to MockONTAPConnection()
        :return: na_ontap_initiator object
        """
        obj = initiator()
        obj.autosupport_log = Mock(return_value=None)
        if kind is None:
            obj.server = MockONTAPConnection()
        else:
            obj.server = MockONTAPConnection(kind=kind)
        return obj

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            initiator()

    def test_get_nonexistent_initiator(self):
        ''' Test if get_initiators returns None for non-existent initiator '''
        data = self.mock_args()
        data['name'] = 'idontexist'
        set_module_args(data)
        result = self.get_initiator_mock_object('initiator').get_initiators()
        assert data['name'] not in result

    def test_get_nonexistent_igroup(self):
        ''' Test if get_initiators returns None for non-existent igroup '''
        data = self.mock_args()
        data['name'] = 'idontexist'
        set_module_args(data)
        result = self.get_initiator_mock_object().get_initiators()
        assert result == []

    def test_get_existing_initiator(self):
        ''' Test if get_initiator returns None for existing initiator '''
        data = self.mock_args()
        set_module_args(data)
        result = self.get_initiator_mock_object(kind='initiator').get_initiators()
        assert data['name'] in result
        assert result == ['init1', 'init2']     # from build_igroup_initiators()

    def test_successful_add(self):
        ''' Test successful add'''
        data = self.mock_args()
        data['name'] = 'iamnew'
        set_module_args(data)
        obj = self.get_initiator_mock_object('initiator')
        with pytest.raises(AnsibleExitJson) as exc:
            current = obj.get_initiators()
            obj.apply()
        assert data['name'] not in current
        assert exc.value.args[0]['changed']

    def test_successful_add_idempotency(self):
        ''' Test successful add idempotency '''
        data = self.mock_args()
        set_module_args(data)
        obj = self.get_initiator_mock_object('initiator')
        with pytest.raises(AnsibleExitJson) as exc:
            current_list = obj.get_initiators()
            obj.apply()
        assert data['name'] in current_list
        assert not exc.value.args[0]['changed']

    def test_successful_remove(self):
        ''' Test successful remove '''
        data = self.mock_args()
        data['state'] = 'absent'
        set_module_args(data)
        obj = self.get_initiator_mock_object('initiator')
        with pytest.raises(AnsibleExitJson) as exc:
            current_list = obj.get_initiators()
            obj.apply()
        assert data['name'] in current_list
        assert exc.value.args[0]['changed']

    def test_successful_remove_idempotency(self):
        ''' Test successful remove idempotency'''
        data = self.mock_args()
        data['state'] = 'absent'
        data['name'] = 'alreadyremoved'
        set_module_args(data)
        obj = self.get_initiator_mock_object('initiator')
        with pytest.raises(AnsibleExitJson) as exc:
            current_list = obj.get_initiators()
            obj.apply()
        assert data['name'] not in current_list
        assert not exc.value.args[0]['changed']



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_system_accprofile
short_description: Configure access profiles for system administrators in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify system feature and accprofile category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    system_accprofile:
        description:
            - Configure access profiles for system administrators.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            admintimeout:
                description:
                    - Administrator timeout for this access profile (0 - 480 min, default = 10, 0 means never timeout).
            admintimeout-override:
                description:
                    - Enable/disable overriding the global administrator idle timeout.
                choices:
                    - enable
                    - disable
            authgrp:
                description:
                    - Administrator access to Users and Devices.
                choices:
                    - none
                    - read
                    - read-write
            comments:
                description:
                    - Comment.
            ftviewgrp:
                description:
                    - FortiView.
                choices:
                    - none
                    - read
                    - read-write
            fwgrp:
                description:
                    - Administrator access to the Firewall configuration.
                choices:
                    - none
                    - read
                    - read-write
                    - custom
            fwgrp-permission:
                description:
                    - Custom firewall permission.
                suboptions:
                    address:
                        description:
                            - Address Configuration.
                        choices:
                            - none
                            - read
                            - read-write
                    policy:
                        description:
                            - Policy Configuration.
                        choices:
                            - none
                            - read
                            - read-write
                    schedule:
                        description:
                            - Schedule Configuration.
                        choices:
                            - none
                            - read
                            - read-write
                    service:
                        description:
                            - Service Configuration.
                        choices:
                            - none
                            - read
                            - read-write
            loggrp:
                description:
                    - Administrator access to Logging and Reporting including viewing log messages.
                choices:
                    - none
                    - read
                    - read-write
                    - custom
            loggrp-permission:
                description:
                    - Custom Log & Report permission.
                suboptions:
                    config:
                        description:
                            - Log & Report configuration.
                        choices:
                            - none
                            - read
                            - read-write
                    data-access:
                        description:
                            - Log & Report Data Access.
                        choices:
                            - none
                            - read
                            - read-write
                    report-access:
                        description:
                            - Log & Report Report Access.
                        choices:
                            - none
                            - read
                            - read-write
                    threat-weight:
                        description:
                            - Log & Report Threat Weight.
                        choices:
                            - none
                            - read
                            - read-write
            name:
                description:
                    - Profile name.
                required: true
            netgrp:
                description:
                    - Network Configuration.
                choices:
                    - none
                    - read
                    - read-write
                    - custom
            netgrp-permission:
                description:
                    - Custom network permission.
                suboptions:
                    cfg:
                        description:
                            - Network Configuration.
                        choices:
                            - none
                            - read
                            - read-write
                    packet-capture:
                        description:
                            - Packet Capture Configuration.
                        choices:
                            - none
                            - read
                            - read-write
                    route-cfg:
                        description:
                            - Router Configuration.
                        choices:
                            - none
                            - read
                            - read-write
            scope:
                description:
                    - "Scope of admin access: global or specific VDOM(s)."
                choices:
                    - vdom
                    - global
            secfabgrp:
                description:
                    - Security Fabric.
                choices:
                    - none
                    - read
                    - read-write
            sysgrp:
                description:
                    - System Configuration.
                choices:
                    - none
                    - read
                    - read-write
                    - custom
            sysgrp-permission:
                description:
                    - Custom system permission.
                suboptions:
                    admin:
                        description:
                            - Administrator Users.
                        choices:
                            - none
                            - read
                            - read-write
                    cfg:
                        description:
                            - System Configuration.
                        choices:
                            - none
                            - read
                            - read-write
                    mnt:
                        description:
                            - Maintenance.
                        choices:
                            - none
                            - read
                            - read-write
                    upd:
                        description:
                            - FortiGuard Updates.
                        choices:
                            - none
                            - read
                            - read-write
            utmgrp:
                description:
                    - Administrator access to Security Profiles.
                choices:
                    - none
                    - read
                    - read-write
                    - custom
            utmgrp-permission:
                description:
                    - Custom Security Profile permissions.
                suboptions:
                    antivirus:
                        description:
                            - Antivirus profiles and settings.
                        choices:
                            - none
                            - read
                            - read-write
                    application-control:
                        description:
                            - Application Control profiles and settings.
                        choices:
                            - none
                            - read
                            - read-write
                    data-loss-prevention:
                        description:
                            - DLP profiles and settings.
                        choices:
                            - none
                            - read
                            - read-write
                    dnsfilter:
                        description:
                            - DNS Filter profiles and settings.
                        choices:
                            - none
                            - read
                            - read-write
                    endpoint-control:
                        description:
                            - FortiClient Profiles.
                        choices:
                            - none
                            - read
                            - read-write
                    icap:
                        description:
                            - ICAP profiles and settings.
                        choices:
                            - none
                            - read
                            - read-write
                    ips:
                        description:
                            - IPS profiles and settings.
                        choices:
                            - none
                            - read
                            - read-write
                    spamfilter:
                        description:
                            - AntiSpam filter and settings.
                        choices:
                            - none
                            - read
                            - read-write
                    voip:
                        description:
                            - VoIP profiles and settings.
                        choices:
                            - none
                            - read
                            - read-write
                    waf:
                        description:
                            - Web Application Firewall profiles and settings.
                        choices:
                            - none
                            - read
                            - read-write
                    webfilter:
                        description:
                            - Web Filter profiles and settings.
                        choices:
                            - none
                            - read
                            - read-write
            vpngrp:
                description:
                    - Administrator access to IPsec, SSL, PPTP, and L2TP VPN.
                choices:
                    - none
                    - read
                    - read-write
            wanoptgrp:
                description:
                    - Administrator access to WAN Opt & Cache.
                choices:
                    - none
                    - read
                    - read-write
            wifi:
                description:
                    - Administrator access to the WiFi controller and Switch controller.
                choices:
                    - none
                    - read
                    - read-write
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure access profiles for system administrators.
    fortios_system_accprofile:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      system_accprofile:
        state: "present"
        admintimeout: "3"
        admintimeout-override: "enable"
        authgrp: "none"
        comments: "<your_own_value>"
        ftviewgrp: "none"
        fwgrp: "none"
        fwgrp-permission:
            address: "none"
            policy: "none"
            schedule: "none"
            service: "none"
        loggrp: "none"
        loggrp-permission:
            config: "none"
            data-access: "none"
            report-access: "none"
            threat-weight: "none"
        name: "default_name_20"
        netgrp: "none"
        netgrp-permission:
            cfg: "none"
            packet-capture: "none"
            route-cfg: "none"
        scope: "vdom"
        secfabgrp: "none"
        sysgrp: "none"
        sysgrp-permission:
            admin: "none"
            cfg: "none"
            mnt: "none"
            upd: "none"
        utmgrp: "none"
        utmgrp-permission:
            antivirus: "none"
            application-control: "none"
            data-loss-prevention: "none"
            dnsfilter: "none"
            endpoint-control: "none"
            icap: "none"
            ips: "none"
            spamfilter: "none"
            voip: "none"
            waf: "none"
            webfilter: "none"
        vpngrp: "none"
        wanoptgrp: "none"
        wifi: "none"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_system_accprofile_data(json):
    option_list = ['admintimeout', 'admintimeout-override', 'authgrp',
                   'comments', 'ftviewgrp', 'fwgrp',
                   'fwgrp-permission', 'loggrp', 'loggrp-permission',
                   'name', 'netgrp', 'netgrp-permission',
                   'scope', 'secfabgrp', 'sysgrp',
                   'sysgrp-permission', 'utmgrp', 'utmgrp-permission',
                   'vpngrp', 'wanoptgrp', 'wifi']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def system_accprofile(data, fos):
    vdom = data['vdom']
    system_accprofile_data = data['system_accprofile']
    flattened_data = flatten_multilists_attributes(system_accprofile_data)
    filtered_data = filter_system_accprofile_data(flattened_data)
    if system_accprofile_data['state'] == "present":
        return fos.set('system',
                       'accprofile',
                       data=filtered_data,
                       vdom=vdom)

    elif system_accprofile_data['state'] == "absent":
        return fos.delete('system',
                          'accprofile',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_system(data, fos):
    login(data)

    if data['system_accprofile']:
        resp = system_accprofile(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "system_accprofile": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "admintimeout": {"required": False, "type": "int"},
                "admintimeout-override": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "authgrp": {"required": False, "type": "str",
                            "choices": ["none", "read", "read-write"]},
                "comments": {"required": False, "type": "str"},
                "ftviewgrp": {"required": False, "type": "str",
                              "choices": ["none", "read", "read-write"]},
                "fwgrp": {"required": False, "type": "str",
                          "choices": ["none", "read", "read-write",
                                      "custom"]},
                "fwgrp-permission": {"required": False, "type": "dict",
                                     "options": {
                                         "address": {"required": False, "type": "str",
                                                     "choices": ["none", "read", "read-write"]},
                                         "policy": {"required": False, "type": "str",
                                                    "choices": ["none", "read", "read-write"]},
                                         "schedule": {"required": False, "type": "str",
                                                      "choices": ["none", "read", "read-write"]},
                                         "service": {"required": False, "type": "str",
                                                     "choices": ["none", "read", "read-write"]}
                                     }},
                "loggrp": {"required": False, "type": "str",
                           "choices": ["none", "read", "read-write",
                                       "custom"]},
                "loggrp-permission": {"required": False, "type": "dict",
                                      "options": {
                                          "config": {"required": False, "type": "str",
                                                     "choices": ["none", "read", "read-write"]},
                                          "data-access": {"required": False, "type": "str",
                                                          "choices": ["none", "read", "read-write"]},
                                          "report-access": {"required": False, "type": "str",
                                                            "choices": ["none", "read", "read-write"]},
                                          "threat-weight": {"required": False, "type": "str",
                                                            "choices": ["none", "read", "read-write"]}
                                      }},
                "name": {"required": True, "type": "str"},
                "netgrp": {"required": False, "type": "str",
                           "choices": ["none", "read", "read-write",
                                       "custom"]},
                "netgrp-permission": {"required": False, "type": "dict",
                                      "options": {
                                          "cfg": {"required": False, "type": "str",
                                                  "choices": ["none", "read", "read-write"]},
                                          "packet-capture": {"required": False, "type": "str",
                                                             "choices": ["none", "read", "read-write"]},
                                          "route-cfg": {"required": False, "type": "str",
                                                        "choices": ["none", "read", "read-write"]}
                                      }},
                "scope": {"required": False, "type": "str",
                          "choices": ["vdom", "global"]},
                "secfabgrp": {"required": False, "type": "str",
                              "choices": ["none", "read", "read-write"]},
                "sysgrp": {"required": False, "type": "str",
                           "choices": ["none", "read", "read-write",
                                       "custom"]},
                "sysgrp-permission": {"required": False, "type": "dict",
                                      "options": {
                                          "admin": {"required": False, "type": "str",
                                                    "choices": ["none", "read", "read-write"]},
                                          "cfg": {"required": False, "type": "str",
                                                  "choices": ["none", "read", "read-write"]},
                                          "mnt": {"required": False, "type": "str",
                                                  "choices": ["none", "read", "read-write"]},
                                          "upd": {"required": False, "type": "str",
                                                  "choices": ["none", "read", "read-write"]}
                                      }},
                "utmgrp": {"required": False, "type": "str",
                           "choices": ["none", "read", "read-write",
                                       "custom"]},
                "utmgrp-permission": {"required": False, "type": "dict",
                                      "options": {
                                          "antivirus": {"required": False, "type": "str",
                                                        "choices": ["none", "read", "read-write"]},
                                          "application-control": {"required": False, "type": "str",
                                                                  "choices": ["none", "read", "read-write"]},
                                          "data-loss-prevention": {"required": False, "type": "str",
                                                                   "choices": ["none", "read", "read-write"]},
                                          "dnsfilter": {"required": False, "type": "str",
                                                        "choices": ["none", "read", "read-write"]},
                                          "endpoint-control": {"required": False, "type": "str",
                                                               "choices": ["none", "read", "read-write"]},
                                          "icap": {"required": False, "type": "str",
                                                   "choices": ["none", "read", "read-write"]},
                                          "ips": {"required": False, "type": "str",
                                                  "choices": ["none", "read", "read-write"]},
                                          "spamfilter": {"required": False, "type": "str",
                                                         "choices": ["none", "read", "read-write"]},
                                          "voip": {"required": False, "type": "str",
                                                   "choices": ["none", "read", "read-write"]},
                                          "waf": {"required": False, "type": "str",
                                                  "choices": ["none", "read", "read-write"]},
                                          "webfilter": {"required": False, "type": "str",
                                                        "choices": ["none", "read", "read-write"]}
                                      }},
                "vpngrp": {"required": False, "type": "str",
                           "choices": ["none", "read", "read-write"]},
                "wanoptgrp": {"required": False, "type": "str",
                              "choices": ["none", "read", "read-write"]},
                "wifi": {"required": False, "type": "str",
                         "choices": ["none", "read", "read-write"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_system(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2017, Jacob McGill (jmcgill298)
# Copyright: (c) 2018, Dag Wieers (dagwieers) <dag@wieers.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: aci_vlan_pool_encap_block
short_description: Manage encap blocks assigned to VLAN pools (fvns:EncapBlk)
description:
- Manage VLAN encap blocks that are assigned to VLAN pools on Cisco ACI fabrics.
version_added: '2.5'
options:
  allocation_mode:
    description:
    - The method used for allocating encaps to resources.
    type: str
    choices: [ dynamic, inherit, static]
    aliases: [ mode ]
  description:
    description:
    - Description for the pool encap block.
    type: str
    aliases: [ descr ]
  pool:
    description:
    - The name of the pool that the encap block should be assigned to.
    type: str
    aliases: [ pool_name ]
  pool_allocation_mode:
    description:
    - The method used for allocating encaps to resources.
    type: str
    choices: [ dynamic, static]
    aliases: [ pool_mode ]
  block_end:
    description:
    - The end of encap block.
    type: int
    aliases: [ end ]
  block_name:
    description:
    - The name to give to the encap block.
    type: str
    aliases: [ name ]
  block_start:
    description:
    - The start of the encap block.
    type: int
    aliases: [ start ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment: aci
notes:
- The C(pool) must exist in order to add or delete a encap block.
seealso:
- module: aci_encap_pool_range
- module: aci_vlan_pool
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fvns:EncapBlk).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Jacob McGill (@jmcgill298)
- Dag Wieers (@dagwieers)
'''

EXAMPLES = r'''
- name: Add a new VLAN encap block
  aci_vlan_pool_encap_block:
    host: apic
    username: admin
    password: SomeSecretPassword
    pool: production
    block_start: 20
    block_end: 50
    state: present
  delegate_to: localhost

- name: Remove a VLAN encap block
  aci_vlan_pool_encap_block:
    host: apic
    username: admin
    password: SomeSecretPassword
    pool: production
    block_start: 20
    block_end: 50
    state: absent
  delegate_to: localhost

- name: Query a VLAN encap block
  aci_vlan_pool_encap_block:
    host: apic
    username: admin
    password: SomeSecretPassword
    pool: production
    block_start: 20
    block_end: 50
    state: query
  delegate_to: localhost
  register: query_result

- name: Query a VLAN pool for encap blocks
  aci_vlan_pool_encap_block:
    host: apic
    username: admin
    password: SomeSecretPassword
    pool: production
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all VLAN encap blocks
  aci_vlan_pool_encap_block:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result
'''

RETURN = r'''
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.aci.aci import ACIModule, aci_argument_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(
        pool=dict(type='str', aliases=['pool_name']),  # Not required for querying all objects
        block_name=dict(type='str', aliases=['name']),  # Not required for querying all objects
        block_end=dict(type='int', aliases=['end']),  # Not required for querying all objects
        block_start=dict(type='int', aliases=["start"]),  # Not required for querying all objects
        allocation_mode=dict(type='str', aliases=['mode'], choices=['dynamic', 'inherit', 'static']),
        description=dict(type='str', aliases=['descr']),
        pool_allocation_mode=dict(type='str', aliases=['pool_mode'], choices=['dynamic', 'static']),
        state=dict(type='str', default='present', choices=['absent', 'present', 'query']),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ['state', 'absent', ['pool', 'block_end', 'block_name', 'block_start']],
            ['state', 'present', ['pool', 'block_end', 'block_name', 'block_start']],
        ],
    )

    allocation_mode = module.params['allocation_mode']
    description = module.params['description']
    pool = module.params['pool']
    pool_allocation_mode = module.params['pool_allocation_mode']
    block_end = module.params['block_end']
    block_name = module.params['block_name']
    block_start = module.params['block_start']
    state = module.params['state']

    if block_end is not None:
        encap_end = 'vlan-{0}'.format(block_end)
    else:
        encap_end = None

    if block_start is not None:
        encap_start = 'vlan-{0}'.format(block_start)
    else:
        encap_start = None

    # Collect proper mo information
    aci_block_mo = 'from-[{0}]-to-[{1}]'.format(encap_start, encap_end)
    pool_name = pool

    # Validate block_end and block_start are valid for its respective encap type
    for encap_id in block_end, block_start:
        if encap_id is not None:
            if not 1 <= encap_id <= 4094:
                module.fail_json(msg="vlan pools must have 'block_start' and 'block_end' values between 1 and 4094")

    if block_end is not None and block_start is not None:
        # Validate block_start is less than block_end
        if block_start > block_end:
            module.fail_json(msg="The 'block_start' must be less than or equal to the 'block_end'")

    elif block_end is None and block_start is None:
        if block_name is None:
            # Reset range managed object to None for aci util to properly handle query
            aci_block_mo = None

    # ACI Pool URL requires the allocation mode (ex: uni/infra/vlanns-[poolname]-static)
    if pool is not None:
        if pool_allocation_mode is not None:
            pool_name = '[{0}]-{1}'.format(pool, pool_allocation_mode)
        else:
            module.fail_json(msg="ACI requires the 'pool_allocation_mode' when 'pool' is provided")

    aci = ACIModule(module)
    aci.construct_url(
        root_class=dict(
            aci_class='fvnsVlanInstP',
            aci_rn='infra/vlanns-{0}'.format(pool_name),
            module_object=pool,
            target_filter={'name': pool},
        ),
        subclass_1=dict(
            aci_class='fvnsEncapBlk',
            aci_rn=aci_block_mo,
            module_object=aci_block_mo,
            target_filter={'from': encap_start, 'to': encap_end, 'name': block_name},
        ),
    )

    aci.get_existing()

    if state == 'present':
        aci.payload(
            aci_class='fvnsEncapBlk',
            class_config={
                "allocMode": allocation_mode,
                "descr": description,
                "from": encap_start,
                "name": block_name,
                "to": encap_end,
            },
        )

        aci.get_diff(aci_class='fvnsEncapBlk')

        aci.post_config()

    elif state == 'absent':
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# FIXME: this style (full module import via from) doesn't work yet from collections
# from ansible_collections.testns.testcoll.plugins.module_utils import secondary
import ansible_collections.testns.testcoll.plugins.module_utils.secondary


def thingtocall():
    return "thingtocall in base called " + ansible_collections.testns.testcoll.plugins.module_utils.secondary.thingtocall()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 Google
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#     Please read more about how to change this file at
#     https://www.github.com/GoogleCloudPlatform/magic-modules
#
# ----------------------------------------------------------------------------

from __future__ import absolute_import, division, print_function

__metaclass__ = type

################################################################################
# Documentation
################################################################################

ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ["preview"], 'supported_by': 'community'}

DOCUMENTATION = '''
---
module: gcp_sql_instance_facts
description:
- Gather facts for GCP Instance
short_description: Gather facts for GCP Instance
version_added: 2.8
author: Google Inc. (@googlecloudplatform)
requirements:
- python >= 2.6
- requests >= 2.18.4
- google-auth >= 1.3.0
options: {}
extends_documentation_fragment: gcp
'''

EXAMPLES = '''
- name: " a instance facts"
  gcp_sql_instance_facts:
    project: test_project
    auth_kind: serviceaccount
    service_account_file: "/tmp/auth.pem"
    state: facts
'''

RETURN = '''
items:
  description: List of items
  returned: always
  type: complex
  contains:
    backendType:
      description:
      - "* FIRST_GEN: First Generation instance. MySQL only."
      - "* SECOND_GEN: Second Generation instance or PostgreSQL instance."
      - "* EXTERNAL: A database server that is not managed by Google."
      returned: success
      type: str
    connectionName:
      description:
      - Connection name of the Cloud SQL instance used in connection strings.
      returned: success
      type: str
    databaseVersion:
      description:
      - The database engine type and version. For First Generation instances, can
        be MYSQL_5_5, or MYSQL_5_6. For Second Generation instances, can be MYSQL_5_6
        or MYSQL_5_7. Defaults to MYSQL_5_6.
      - 'PostgreSQL instances: POSTGRES_9_6 The databaseVersion property can not be
        changed after instance creation.'
      returned: success
      type: str
    failoverReplica:
      description:
      - The name and status of the failover replica. This property is applicable only
        to Second Generation instances.
      returned: success
      type: complex
      contains:
        available:
          description:
          - The availability status of the failover replica. A false status indicates
            that the failover replica is out of sync. The master can only failover
            to the failover replica when the status is true.
          returned: success
          type: bool
        name:
          description:
          - The name of the failover replica. If specified at instance creation, a
            failover replica is created for the instance. The name doesn't include
            the project ID. This property is applicable only to Second Generation
            instances.
          returned: success
          type: str
    instanceType:
      description:
      - The instance type. This can be one of the following.
      - "* CLOUD_SQL_INSTANCE: A Cloud SQL instance that is not replicating from a
        master."
      - "* ON_PREMISES_INSTANCE: An instance running on the customer's premises."
      - "* READ_REPLICA_INSTANCE: A Cloud SQL instance configured as a read-replica."
      returned: success
      type: str
    ipAddresses:
      description:
      - The assigned IP addresses for the instance.
      returned: success
      type: complex
      contains:
        ipAddress:
          description:
          - The IP address assigned.
          returned: success
          type: str
        timeToRetire:
          description:
          - The due time for this IP to be retired in RFC 3339 format, for example
            2012-11-15T16:19:00.094Z. This field is only available when the IP is
            scheduled to be retired.
          returned: success
          type: str
        type:
          description:
          - The type of this IP address. A PRIMARY address is an address that can
            accept incoming connections. An OUTGOING address is the source address
            of connections originating from the instance, if supported.
          returned: success
          type: str
    ipv6Address:
      description:
      - The IPv6 address assigned to the instance. This property is applicable only
        to First Generation instances.
      returned: success
      type: str
    masterInstanceName:
      description:
      - The name of the instance which will act as master in the replication setup.
      returned: success
      type: str
    maxDiskSize:
      description:
      - The maximum disk size of the instance in bytes.
      returned: success
      type: int
    name:
      description:
      - Name of the Cloud SQL instance. This does not include the project ID.
      returned: success
      type: str
    region:
      description:
      - The geographical region. Defaults to us-central or us-central1 depending on
        the instance type (First Generation or Second Generation/PostgreSQL).
      returned: success
      type: str
    replicaConfiguration:
      description:
      - Configuration specific to failover replicas and read replicas.
      returned: success
      type: complex
      contains:
        failoverTarget:
          description:
          - Specifies if the replica is the failover target. If the field is set to
            true the replica will be designated as a failover replica.
          - In case the master instance fails, the replica instance will be promoted
            as the new master instance.
          - Only one replica can be specified as failover target, and the replica
            has to be in different zone with the master instance.
          returned: success
          type: bool
        mysqlReplicaConfiguration:
          description:
          - MySQL specific configuration when replicating from a MySQL on-premises
            master. Replication configuration information such as the username, password,
            certificates, and keys are not stored in the instance metadata. The configuration
            information is used only to set up the replication connection and is stored
            by MySQL in a file named master.info in the data directory.
          returned: success
          type: complex
          contains:
            caCertificate:
              description:
              - PEM representation of the trusted CA's x509 certificate.
              returned: success
              type: str
            clientCertificate:
              description:
              - PEM representation of the slave's x509 certificate .
              returned: success
              type: str
            clientKey:
              description:
              - PEM representation of the slave's private key. The corresponding public
                key is encoded in the client's certificate.
              returned: success
              type: str
            connectRetryInterval:
              description:
              - Seconds to wait between connect retries. MySQL's default is 60 seconds.
              returned: success
              type: int
            dumpFilePath:
              description:
              - Path to a SQL dump file in Google Cloud Storage from which the slave
                instance is to be created. The URI is in the form gs://bucketName/fileName.
                Compressed gzip files (.gz) are also supported. Dumps should have
                the binlog co-ordinates from which replication should begin. This
                can be accomplished by setting --master-data to 1 when using mysqldump.
              returned: success
              type: str
            masterHeartbeatPeriod:
              description:
              - Interval in milliseconds between replication heartbeats.
              returned: success
              type: int
            password:
              description:
              - The password for the replication connection.
              returned: success
              type: str
            sslCipher:
              description:
              - A list of permissible ciphers to use for SSL encryption.
              returned: success
              type: str
            username:
              description:
              - The username for the replication connection.
              returned: success
              type: str
            verifyServerCertificate:
              description:
              - Whether or not to check the master's Common Name value in the certificate
                that it sends during the SSL handshake.
              returned: success
              type: bool
        replicaNames:
          description:
          - The replicas of the instance.
          returned: success
          type: list
        serviceAccountEmailAddress:
          description:
          - The service account email address assigned to the instance. This property
            is applicable only to Second Generation instances.
          returned: success
          type: str
    settings:
      description:
      - The user settings.
      returned: success
      type: complex
      contains:
        ipConfiguration:
          description:
          - The settings for IP Management. This allows to enable or disable the instance
            IP and manage which external networks can connect to the instance. The
            IPv4 address cannot be disabled for Second Generation instances.
          returned: success
          type: complex
          contains:
            ipv4Enabled:
              description:
              - Whether the instance should be assigned an IP address or not.
              returned: success
              type: bool
            authorizedNetworks:
              description:
              - The list of external networks that are allowed to connect to the instance
                using the IP. In CIDR notation, also known as 'slash' notation (e.g.
                192.168.100.0/24).
              returned: success
              type: complex
              contains:
                expirationTime:
                  description:
                  - The time when this access control entry expires in RFC 3339 format,
                    for example 2012-11-15T16:19:00.094Z.
                  returned: success
                  type: str
                name:
                  description:
                  - An optional label to identify this entry.
                  returned: success
                  type: str
                value:
                  description:
                  - The whitelisted value for the access control list. For example,
                    to grant access to a client from an external IP (IPv4 or IPv6)
                    address or subnet, use that address or subnet here.
                  returned: success
                  type: str
            requireSsl:
              description:
              - Whether the mysqld should default to 'REQUIRE X509' for users connecting
                over IP.
              returned: success
              type: bool
        tier:
          description:
          - The tier or machine type for this instance, for example db-n1-standard-1.
            For MySQL instances, this field determines whether the instance is Second
            Generation (recommended) or First Generation.
          returned: success
          type: str
        availabilityType:
          description:
          - The availabilityType define if your postgres instance is run zonal or
            regional.
          returned: success
          type: str
        backupConfiguration:
          description:
          - The daily backup configuration for the instance.
          returned: success
          type: complex
          contains:
            enabled:
              description:
              - Enable Autobackup for your instance.
              returned: success
              type: bool
            binaryLogEnabled:
              description:
              - Whether binary log is enabled. If backup configuration is disabled,
                binary log must be disabled as well. MySQL only.
              returned: success
              type: bool
            startTime:
              description:
              - Define the backup start time in UTC (HH:MM) .
              returned: success
              type: str
        settingsVersion:
          description:
          - The version of instance settings. This is a required field for update
            method to make sure concurrent updates are handled properly. During update,
            use the most recent settingsVersion value for this instance and do not
            try to update this value.
          returned: success
          type: int
'''

################################################################################
# Imports
################################################################################
from ansible.module_utils.gcp_utils import navigate_hash, GcpSession, GcpModule, GcpRequest
import json

################################################################################
# Main
################################################################################


def main():
    module = GcpModule(argument_spec=dict())

    if not module.params['scopes']:
        module.params['scopes'] = ['https://www.googleapis.com/auth/sqlservice.admin']

    items = fetch_list(module, collection(module))
    if items.get('items'):
        items = items.get('items')
    else:
        items = []
    return_value = {'items': items}
    module.exit_json(**return_value)


def collection(module):
    return "https://www.googleapis.com/sql/v1beta4/projects/{project}/instances".format(**module.params)


def fetch_list(module, link):
    auth = GcpSession(module, 'sql')
    response = auth.get(link)
    return return_if_object(module, response)


def return_if_object(module, response):
    # If not found, return nothing.
    if response.status_code == 404:
        return None

    # If no content, return nothing.
    if response.status_code == 204:
        return None

    try:
        module.raise_for_status(response)
        result = response.json()
    except getattr(json.decoder, 'JSONDecodeError', ValueError) as inst:
        module.fail_json(msg="Invalid JSON response with error: %s" % inst)

    if navigate_hash(result, ['error', 'errors']):
        module.fail_json(msg=navigate_hash(result, ['error', 'errors']))

    return result


if __name__ == "__main__":
    main()



# Copyright: (c) 2018, Abhijeet Kasurde <akasurde@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import pytest

from ansible import context
from ansible.cli.adhoc import AdHocCLI, display
from ansible.errors import AnsibleOptionsError


def test_parse():
    """ Test adhoc parse"""
    adhoc_cli = AdHocCLI([])
    with pytest.raises(AnsibleOptionsError) as exec_info:
        adhoc_cli.parse()
    assert "Missing target hosts" == str(exec_info.value)


def test_with_command():
    """ Test simple adhoc command"""
    module_name = 'command'
    adhoc_cli = AdHocCLI(args=['-m', module_name, '-vv'])
    adhoc_cli.parse()
    assert context.CLIARGS['module_name'] == module_name
    assert display.verbosity == 2


def test_with_extra_parameters():
    """ Test extra parameters"""
    adhoc_cli = AdHocCLI(args=['-m', 'command', 'extra_parameters'])
    with pytest.raises(AnsibleOptionsError) as exec_info:
        adhoc_cli.parse()
    assert "Extraneous options or arguments" == str(exec_info.value)


def test_simple_command():
    """ Test valid command and its run"""
    adhoc_cli = AdHocCLI(['/bin/ansible', '-m', 'command', 'localhost', '-a', 'echo "hi"'])
    adhoc_cli.parse()
    ret = adhoc_cli.run()
    assert ret == 0


def test_no_argument():
    """ Test no argument command"""
    adhoc_cli = AdHocCLI(['/bin/ansible', '-m', 'command', 'localhost'])
    adhoc_cli.parse()
    with pytest.raises(AnsibleOptionsError) as exec_info:
        adhoc_cli.run()
    assert 'No argument passed to command module' == str(exec_info.value)


def test_did_you_mean_playbook():
    """ Test adhoc with yml file as argument parameter"""
    adhoc_cli = AdHocCLI(['/bin/ansible', '-m', 'command', 'localhost.yml'])
    adhoc_cli.parse()
    with pytest.raises(AnsibleOptionsError) as exec_info:
        adhoc_cli.run()
    assert 'No argument passed to command module (did you mean to run ansible-playbook?)' == str(exec_info.value)


def test_play_ds_positive():
    """ Test _play_ds"""
    adhoc_cli = AdHocCLI(args=['/bin/ansible', 'localhost', '-m', 'command'])
    adhoc_cli.parse()
    ret = adhoc_cli._play_ds('command', 10, 2)
    assert ret['name'] == 'Ansible Ad-Hoc'
    assert ret['tasks'] == [{'action': {'module': 'command', 'args': {}}, 'async_val': 10, 'poll': 2}]


def test_play_ds_with_include_role():
    """ Test include_role command with poll"""
    adhoc_cli = AdHocCLI(args=['/bin/ansible', 'localhost', '-m', 'include_role'])
    adhoc_cli.parse()
    ret = adhoc_cli._play_ds('include_role', None, 2)
    assert ret['name'] == 'Ansible Ad-Hoc'
    assert ret['gather_facts'] == 'no'


def test_run_import_playbook():
    """ Test import_playbook which is not allowed with ad-hoc command"""
    import_playbook = 'import_playbook'
    adhoc_cli = AdHocCLI(args=['/bin/ansible', '-m', import_playbook, 'localhost'])
    adhoc_cli.parse()
    with pytest.raises(AnsibleOptionsError) as exec_info:
        adhoc_cli.run()
    assert context.CLIARGS['module_name'] == import_playbook
    assert "'%s' is not a valid action for ad-hoc commands" % import_playbook == str(exec_info.value)



# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from units.compat.mock import patch
from ansible.modules.network.vyos import vyos_static_route
from units.modules.utils import set_module_args
from .vyos_module import TestVyosModule, load_fixture


class TestVyosStaticRouteModule(TestVyosModule):

    module = vyos_static_route

    def setUp(self):
        super(TestVyosStaticRouteModule, self).setUp()

        self.mock_get_config = patch('ansible.modules.network.vyos.vyos_static_route.get_config')
        self.get_config = self.mock_get_config.start()

        self.mock_load_config = patch('ansible.modules.network.vyos.vyos_static_route.load_config')
        self.load_config = self.mock_load_config.start()

    def tearDown(self):
        super(TestVyosStaticRouteModule, self).tearDown()

        self.mock_get_config.stop()
        self.mock_load_config.stop()

    def load_fixtures(self, commands=None, transport='cli'):
        self.load_config.return_value = dict(diff=None, session='session')

    def test_vyos_static_route_present(self):
        set_module_args(dict(prefix='172.26.0.0/16', next_hop='172.26.4.1', admin_distance='1'))
        result = self.execute_module(changed=True)
        self.assertEqual(result['commands'],
                         ['set protocols static route 172.26.0.0/16 next-hop 172.26.4.1 distance 1'])



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2012, Dag Wieers <dag@wieers.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['stableinterface'],
                    'supported_by': 'core'}

DOCUMENTATION = r'''
---
module: fail
short_description: Fail with custom message
description:
- This module fails the progress with a custom message.
- It can be useful for bailing out when a certain condition is met using C(when).
- This module is also supported for Windows targets.
version_added: "0.8"
options:
  msg:
    description:
    - The customized message used for failing execution.
    - If omitted, fail will simply bail out with a generic message.
    type: str
    default: Failed as requested from task
notes:
    - This module is also supported for Windows targets.
seealso:
- module: assert
- module: debug
- module: meta
author:
- Dag Wieers (@dagwieers)
'''

EXAMPLES = r'''
# Example playbook using fail and when together
- fail:
    msg: The system may not be provisioned according to the CMDB status.
  when: cmdb_status != "to-be-staged"
'''



# -*- coding: utf-8 -*-

import os
import sys

from units.compat import unittest
from ansible.module_utils.hwc_utils import (navigate_hash,
                                            remove_empty_from_dict,
                                            remove_nones_from_dict,
                                            replace_resource_dict)


class HwcUtilsTestCase(unittest.TestCase):
    def test_navigate_hash(self):
        value = {
            'foo': {
                'quiet': {
                    'tree': 'test'
                },
            }
        }

        self.assertEquals(navigate_hash(value, ["foo", "quiet", "tree"]),
                          "test")

        self.assertEquals(navigate_hash(value, ["foo", "q", "tree"], 123),
                          123)

        self.assertIsNone(navigate_hash(value, [], 123))

    def test_remove_empty_from_dict(self):
        value = {
            'foo': {
                'quiet': {
                    'tree': 'test',
                    'tree1': [
                        None,
                        {},
                        [],
                        'test'
                    ],
                    'tree2': {},
                    'tree3': []
                },
            },
            'foo1': [],
            'foo2': {},
            'foo3': None,
        }

        expect = {
            'foo': {
                'quiet': {
                    'tree': 'test',
                    'tree1': [
                        'test'
                    ]
                },
            },
        }

        self.assertEqual(remove_empty_from_dict(value), expect)

    def test_remove_nones_from_dict(self):
        value = {
            'foo': {
                'quiet': {
                    'tree': 'test',
                    'tree1': [
                        None,
                        {},
                        [],
                        'test'
                    ],
                    'tree2': {},
                    'tree3': []
                },
            },
            'foo1': [],
            'foo2': {},
            'foo3': None,
        }

        expect = {
            'foo': {
                'quiet': {
                    'tree': 'test',
                    'tree1': [
                        {},
                        [],
                        'test'
                    ],
                    'tree2': {},
                    'tree3': []
                },
            },
            'foo1': [],
            'foo2': {},
        }

        self.assertEqual(remove_nones_from_dict(value), expect)

    def test_replace_resource_dict(self):
        self.assertEqual(replace_resource_dict({'foo': 'quiet'}, 'foo'), 'quiet')

        self.assertEqual(replace_resource_dict({}, 'foo'), {})

        self.assertEqual(replace_resource_dict([[{'foo': 'quiet'}]], 'foo'),
                         [['quiet']])



# (c) 2019, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
''' unit test for Ansible module: na_elementsw_initiators.py '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

if not netapp_utils.has_sf_sdk():
    pytestmark = pytest.mark.skip('skipping as missing required SolidFire Python SDK')

from ansible.modules.storage.netapp.na_elementsw_initiators \
    import ElementSWInitiators as my_module  # module under test


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockSFConnection(object):
    ''' mock connection to ElementSW host '''

    class Bunch(object):  # pylint: disable=too-few-public-methods
        ''' create object with arbitrary attributes '''
        def __init__(self, **kw):
            ''' called with (k1=v1, k2=v2), creates obj.k1, obj.k2 with values v1, v2 '''
            setattr(self, '__dict__', kw)

    class Initiator(object):
        def __init__(self, entries):
            self.__dict__.update(entries)

    def list_initiators(self):
        ''' build initiator Obj '''
        all_initiators = {
            "initiators": [{
                "initiator_name": "a",
                "initiator_id": 13,
                "alias": "a2",
                "attributes": {"key": "value"}
            }]
        }
        return json.loads(json.dumps(all_initiators), object_hook=self.Initiator)

    def create_initiators(self, *args, **kwargs):  # pylint: disable=unused-argument
        ''' mock method '''
        pass

    def delete_initiators(self, *args, **kwargs):  # pylint: disable=unused-argument
        ''' mock method '''
        pass

    def modify_initiators(self, *args, **kwargs):  # pylint: disable=unused-argument
        ''' mock method '''
        pass


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)

    def set_default_args(self):
        return dict({
            'hostname': '10.253.168.129',
            'username': 'namburu',
            'password': 'SFlab1234',
        })

    @patch('ansible.module_utils.netapp.create_sf_connection')
    def test_module_fail_when_required_args_missing(self, mock_create_sf_connection):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            my_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    @patch('ansible.module_utils.netapp.create_sf_connection')
    def test_create_initiators(self, mock_create_sf_connection):
        ''' test if create initiator is called '''
        module_args = {}
        module_args.update(self.set_default_args())
        initiator_dict = {
            "state": "present",
            "initiators": [{
                "name": "newinitiator1",
                "alias": "newinitiator1alias",
                "attributes": {"key1": "value1"}
            }]
        }
        module_args.update(initiator_dict)
        set_module_args(module_args)
        mock_create_sf_connection.return_value = MockSFConnection()
        my_obj = my_module()
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        print('Info: test_create_initiators: %s' % repr(exc.value))
        assert exc.value.args[0]['changed']

    @patch('ansible.module_utils.netapp.create_sf_connection')
    def test_delete_initiators(self, mock_create_sf_connection):
        ''' test if delete initiator is called '''
        module_args = {}
        module_args.update(self.set_default_args())
        initiator_dict = {
            "state": "absent",
            "initiators": [{
                "name": "a"
            }]
        }
        module_args.update(initiator_dict)
        set_module_args(module_args)
        mock_create_sf_connection.return_value = MockSFConnection()
        my_obj = my_module()
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        print('Info: test_delete_initiators: %s' % repr(exc.value))
        assert exc.value.args[0]['changed']

    @patch('ansible.module_utils.netapp.create_sf_connection')
    def test_modify_initiators(self, mock_create_sf_connection):
        ''' test if modify initiator is called '''
        module_args = {}
        module_args.update(self.set_default_args())
        initiator_dict = {
            "state": "present",
            "initiators": [{
                "initiator_name": "a",
                "alias": "a3",
                "attributes": {"key": "value"}
            }]
        }
        module_args.update(initiator_dict)
        set_module_args(module_args)
        mock_create_sf_connection.return_value = MockSFConnection()
        my_obj = my_module()
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        print('Info: test_modify_initiators: %s' % repr(exc.value))
        assert exc.value.args[0]['changed']



#!/usr/bin/python
# Copyright: (c) 2018, Pluribus Networks
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = """
---
module: pn_vrouter_interface_ip
author: "Pluribus Networks (@rajaspachipulusu17)"
version_added: "2.8"
short_description: CLI command to add/remove vrouter-interface-ip
description:
  - This module can be used to add an IP address on interface from a vRouter
    or remove an IP address on interface from a vRouter.
options:
  pn_cliswitch:
    description:
      - Target switch to run the CLI on.
    required: false
    type: str
  state:
    description:
      - State the action to perform. Use C(present) to addvrouter-interface-ip
        and C(absent) to remove vrouter-interface-ip.
    required: true
    type: str
    choices: ['present', 'absent']
  pn_bd:
    description:
      - interface Bridge Domain.
    required: false
    type: str
  pn_netmask:
    description:
      - netmask.
    required: false
    type: str
  pn_vnet:
    description:
      - interface VLAN VNET.
    required: false
    type: str
  pn_ip:
    description:
      - IP address.
    required: false
    type: str
  pn_nic:
    description:
      - virtual NIC assigned to interface.
    required: false
    type: str
  pn_vrouter_name:
    description:
      - name of service config.
    required: false
    type: str
"""

EXAMPLES = """
- name: Add vrouter interface to nic
  pn_vrouter_interface_ip:
    state: "present"
    pn_cliswitch: "sw01"
    pn_vrouter_name: "foo-vrouter"
    pn_ip: "2620:0:1651:1::30"
    pn_netmask: "127"
    pn_nic: "eth0.4092"

- name: Remove vrouter interface to nic
  pn_vrouter_interface_ip:
    state: "absent"
    pn_cliswitch: "sw01"
    pn_vrouter_name: "foo-vrouter"
    pn_ip: "2620:0:1651:1::30"
    pn_nic: "eth0.4092"
"""

RETURN = """
command:
  description: the CLI command run on the target node.
  returned: always
  type: str
stdout:
  description: set of responses from the vrouter-interface-ip command.
  returned: always
  type: list
stderr:
  description: set of error responses from the vrouter-interface-ip command.
  returned: on error
  type: list
changed:
  description: indicates whether the CLI caused changes on the target.
  returned: always
  type: bool
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.netvisor.pn_nvos import pn_cli, run_cli
from ansible.module_utils.network.netvisor.netvisor import run_commands


def check_cli(module, cli):
    """
    This method checks if vRouter exists on the target node.
    This method also checks for idempotency using the vrouter-interface-show
    command.
    If the given vRouter exists, return VROUTER_EXISTS as True else False.

    If an interface with the given ip exists on the given vRouter,
    return INTERFACE_EXISTS as True else False. This is required for
    vrouter-interface-add.

    If nic_str exists on the given vRouter, return NIC_EXISTS as True else
    False. This is required for vrouter-interface-remove.

    :param module: The Ansible module to fetch input parameters
    :param cli: The CLI string
    :return Booleans: VROUTER_EXISTS, INTERFACE_EXISTS, NIC_EXISTS
    """
    vrouter_name = module.params['pn_vrouter_name']
    interface_ip = module.params['pn_ip']
    nic_str = module.params['pn_nic']

    # Check for vRouter
    check_vrouter = cli + ' vrouter-show name %s format name no-show-headers ' % vrouter_name
    out = run_commands(module, check_vrouter)[1]
    out = out.split()

    VROUTER_EXISTS = True if vrouter_name in out[-1] else False

    if interface_ip:
        # Check for interface and VRRP and fetch nic for VRRP
        show = cli + ' vrouter-interface-show vrouter-name %s ' % vrouter_name
        show += 'ip2 %s format ip2,nic no-show-headers' % interface_ip
        out = run_commands(module, show)[1]

        if out and interface_ip in out.split(' ')[-2]:
            INTERFACE_EXISTS = True
        else:
            INTERFACE_EXISTS = False

    if nic_str:
        # Check for nic
        show = cli + ' vrouter-interface-show vrouter-name %s ' % vrouter_name
        show += ' format nic no-show-headers'
        out = run_commands(module, show)[1]
        out = out.split()

        NIC_EXISTS = True if nic_str in out else False

    return VROUTER_EXISTS, INTERFACE_EXISTS, NIC_EXISTS


def main():
    """ This section is for arguments parsing """

    state_map = dict(
        present='vrouter-interface-ip-add',
        absent='vrouter-interface-ip-remove'
    )

    module = AnsibleModule(
        argument_spec=dict(
            pn_cliswitch=dict(required=False, type='str'),
            state=dict(required=True, type='str',
                       choices=state_map.keys()),
            pn_bd=dict(required=False, type='str'),
            pn_netmask=dict(required=False, type='str'),
            pn_vnet=dict(required=False, type='str'),
            pn_ip=dict(required=False, type='str'),
            pn_nic=dict(required=False, type='str'),
            pn_vrouter_name=dict(required=False, type='str'),
        ),
        required_if=(
            ["state", "present", ["pn_vrouter_name", "pn_nic", "pn_ip", "pn_netmask"]],
            ["state", "absent", ["pn_vrouter_name", "pn_nic", "pn_ip"]]
        ),
    )

    # Accessing the arguments
    cliswitch = module.params['pn_cliswitch']
    state = module.params['state']
    bd = module.params['pn_bd']
    netmask = module.params['pn_netmask']
    vnet = module.params['pn_vnet']
    ip = module.params['pn_ip']
    nic = module.params['pn_nic']
    vrouter_name = module.params['pn_vrouter_name']

    command = state_map[state]

    # Building the CLI command string
    cli = pn_cli(module, cliswitch)

    VROUTER_EXISTS, INTERFACE_EXISTS, NIC_EXISTS = check_cli(module, cli)

    if VROUTER_EXISTS is False:
        module.fail_json(
            failed=True,
            msg='vRouter %s does not exist' % vrouter_name
        )

    if NIC_EXISTS is False:
        module.fail_json(
            failed=True,
            msg='vRouter with nic %s does not exist' % nic
        )

    cli += ' %s vrouter-name %s ' % (command, vrouter_name)

    if command == 'vrouter-interface-ip-add':
        if INTERFACE_EXISTS is True:
            module.exit_json(
                skipped=True,
                msg='vRouter with interface %s exist' % ip
            )
        cli += ' nic %s ip %s ' % (nic, ip)

        if bd:
            cli += ' bd ' + bd
        if netmask:
            cli += ' netmask ' + netmask
        if vnet:
            cli += ' vnet ' + vnet

    if command == 'vrouter-interface-ip-remove':
        if INTERFACE_EXISTS is False:
            module.exit_json(
                skipped=True,
                msg='vRouter with interface %s does not exist' % ip
            )
        if nic:
            cli += ' nic %s ' % nic
        if ip:
            cli += ' ip %s ' % ip.split('/')[0]

    run_cli(module, cli, state_map)


if __name__ == '__main__':
    main()



''' unit tests ONTAP Ansible module: na_ontap_cluster_peer '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_cluster_peer \
    import NetAppONTAPClusterPeer as my_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, parm1=None):
        ''' save arguments '''
        self.type = kind
        self.data = parm1
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.type == 'cluster_peer':
            xml = self.build_cluster_peer_info(self.data)
        self.xml_out = xml
        return xml

    @staticmethod
    def build_cluster_peer_info(parm1):
        ''' build xml data for vserser-info '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'num-records': 1,
            'attributes-list': {
                'cluster-peer-info': {
                    'cluster-name': parm1['dest_cluster_name'],
                    'peer-addresses': parm1['dest_intercluster_lifs']
                }
            }
        }
        xml.translate_struct(attributes)
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.server = MockONTAPConnection()
        self.mock_cluster_peer = {
            'source_intercluster_lifs': '1.2.3.4,1.2.3.5',
            'dest_intercluster_lifs': '1.2.3.6,1.2.3.7',
            'passphrase': 'netapp123',
            'dest_hostname': '10.20.30.40',
            'dest_cluster_name': 'cluster2',
            'hostname': 'hostname',
            'username': 'username',
            'password': 'password',

        }

    def mock_args(self):
        return {
            'source_intercluster_lifs': self.mock_cluster_peer['source_intercluster_lifs'],
            'dest_intercluster_lifs': self.mock_cluster_peer['dest_intercluster_lifs'],
            'passphrase': self.mock_cluster_peer['passphrase'],
            'dest_hostname': self.mock_cluster_peer['dest_hostname'],
            'dest_cluster_name': 'cluster2',
            'hostname': 'hostname',
            'username': 'username',
            'password': 'password',
        }

    def get_cluster_peer_mock_object(self, kind=None):
        """
        Helper method to return an na_ontap_cluster_peer object
        :param kind: passes this param to MockONTAPConnection()
        :return: na_ontap_cluster_peer object
        """
        cluster_peer_obj = my_module()
        cluster_peer_obj.asup_log_for_cserver = Mock(return_value=None)
        cluster_peer_obj.cluster = Mock()
        cluster_peer_obj.cluster.invoke_successfully = Mock()
        if kind is None:
            cluster_peer_obj.server = MockONTAPConnection()
            cluster_peer_obj.dest_server = MockONTAPConnection()
        else:
            cluster_peer_obj.server = MockONTAPConnection(kind=kind, parm1=self.mock_cluster_peer)
            cluster_peer_obj.dest_server = MockONTAPConnection(kind=kind, parm1=self.mock_cluster_peer)
        return cluster_peer_obj

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            my_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    @patch('ansible.modules.storage.netapp.na_ontap_cluster_peer.NetAppONTAPClusterPeer.cluster_peer_get')
    def test_successful_create(self, cluster_peer_get):
        ''' Test successful create '''
        set_module_args(self.mock_args())
        cluster_peer_get.side_effect = [
            None,
            None
        ]
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_cluster_peer_mock_object().apply()
        assert exc.value.args[0]['changed']

    @patch('ansible.modules.storage.netapp.na_ontap_cluster_peer.NetAppONTAPClusterPeer.cluster_peer_get')
    def test_create_idempotency(self, cluster_peer_get):
        ''' Test create idempotency '''
        set_module_args(self.mock_args())
        current1 = {
            'cluster_name': 'cluster1',
            'peer-addresses': '1.2.3.6,1.2.3.7'
        }
        current2 = {
            'cluster_name': 'cluster2',
            'peer-addresses': '1.2.3.4,1.2.3.5'
        }
        cluster_peer_get.side_effect = [
            current1,
            current2
        ]
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_cluster_peer_mock_object('cluster_peer').apply()
        assert not exc.value.args[0]['changed']

    @patch('ansible.modules.storage.netapp.na_ontap_cluster_peer.NetAppONTAPClusterPeer.cluster_peer_get')
    def test_successful_delete(self, cluster_peer_get):
        ''' Test delete existing interface '''
        data = self.mock_args()
        data['state'] = 'absent'
        data['source_cluster_name'] = 'cluster1'
        set_module_args(data)
        current1 = {
            'cluster_name': 'cluster1',
            'peer-addresses': '1.2.3.6,1.2.3.7'
        }
        current2 = {
            'cluster_name': 'cluster2',
            'peer-addresses': '1.2.3.4,1.2.3.5'
        }
        cluster_peer_get.side_effect = [
            current1,
            current2
        ]
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_cluster_peer_mock_object('cluster_peer').apply()
        assert exc.value.args[0]['changed']

    @patch('ansible.modules.storage.netapp.na_ontap_cluster_peer.NetAppONTAPClusterPeer.cluster_peer_get')
    def test_delete_idempotency(self, cluster_peer_get):
        ''' Test delete idempotency '''
        data = self.mock_args()
        data['state'] = 'absent'
        data['source_cluster_name'] = 'cluster2'
        set_module_args(data)
        cluster_peer_get.side_effect = [
            None,
            None
        ]
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_cluster_peer_mock_object().apply()
        assert not exc.value.args[0]['changed']



# -*- coding: utf-8 -*-
#
# Copyright: (c) 2017, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import json
import pytest
import sys

if sys.version_info < (2, 7):
    pytestmark = pytest.mark.skip("F5 Ansible modules require Python >= 2.7")

from ansible.module_utils.basic import AnsibleModule

try:
    from library.modules.bigip_smtp import ApiParameters
    from library.modules.bigip_smtp import ModuleParameters
    from library.modules.bigip_smtp import ModuleManager
    from library.modules.bigip_smtp import ArgumentSpec

    # In Ansible 2.8, Ansible changed import paths.
    from test.units.compat import unittest
    from test.units.compat.mock import Mock
    from test.units.compat.mock import patch

    from test.units.modules.utils import set_module_args
except ImportError:
    from ansible.modules.network.f5.bigip_smtp import ApiParameters
    from ansible.modules.network.f5.bigip_smtp import ModuleParameters
    from ansible.modules.network.f5.bigip_smtp import ModuleManager
    from ansible.modules.network.f5.bigip_smtp import ArgumentSpec

    # Ansible 2.8 imports
    from units.compat import unittest
    from units.compat.mock import Mock
    from units.compat.mock import patch

    from units.modules.utils import set_module_args


fixture_path = os.path.join(os.path.dirname(__file__), 'fixtures')
fixture_data = {}


def load_fixture(name):
    path = os.path.join(fixture_path, name)

    if path in fixture_data:
        return fixture_data[path]

    with open(path) as f:
        data = f.read()

    try:
        data = json.loads(data)
    except Exception:
        pass

    fixture_data[path] = data
    return data


class TestParameters(unittest.TestCase):
    def test_module_parameters(self):
        args = dict(
            name='foo',
            smtp_server='1.1.1.1',
            smtp_server_port='25',
            smtp_server_username='admin',
            smtp_server_password='password',
            local_host_name='smtp.mydomain.com',
            encryption='tls',
            update_password='always',
            from_address='no-reply@mydomain.com',
            authentication=True,
        )

        p = ModuleParameters(params=args)
        assert p.name == 'foo'
        assert p.smtp_server == '1.1.1.1'
        assert p.smtp_server_port == 25
        assert p.smtp_server_username == 'admin'
        assert p.smtp_server_password == 'password'
        assert p.local_host_name == 'smtp.mydomain.com'
        assert p.encryption == 'tls'
        assert p.update_password == 'always'
        assert p.from_address == 'no-reply@mydomain.com'
        assert p.authentication_disabled is None
        assert p.authentication_enabled is True

    def test_api_parameters(self):
        p = ApiParameters(params=load_fixture('load_sys_smtp_server.json'))
        assert p.name == 'foo'
        assert p.smtp_server == 'mail.foo.bar'
        assert p.smtp_server_port == 465
        assert p.smtp_server_username == 'admin'
        assert p.smtp_server_password == '$M$Ch$this-is-encrypted=='
        assert p.local_host_name == 'mail-host.foo.bar'
        assert p.encryption == 'ssl'
        assert p.from_address == 'no-reply@foo.bar'
        assert p.authentication_disabled is None
        assert p.authentication_enabled is True


class TestManager(unittest.TestCase):

    def setUp(self):
        self.spec = ArgumentSpec()

    def test_create_monitor(self, *args):
        set_module_args(dict(
            name='foo',
            smtp_server='1.1.1.1',
            smtp_server_port='25',
            smtp_server_username='admin',
            smtp_server_password='password',
            local_host_name='smtp.mydomain.com',
            encryption='tls',
            update_password='always',
            from_address='no-reply@mydomain.com',
            authentication=True,
            partition='Common',
            provider=dict(
                server='localhost',
                password='password',
                user='admin'
            )
        ))

        module = AnsibleModule(
            argument_spec=self.spec.argument_spec,
            supports_check_mode=self.spec.supports_check_mode
        )

        # Override methods in the specific type of manager
        mm = ModuleManager(module=module)
        mm.exists = Mock(side_effect=[False, True])
        mm.create_on_device = Mock(return_value=True)

        results = mm.exec_module()

        assert results['changed'] is True
        assert results['encryption'] == 'tls'
        assert results['smtp_server'] == '1.1.1.1'
        assert results['smtp_server_port'] == 25
        assert results['local_host_name'] == 'smtp.mydomain.com'
        assert results['authentication'] is True
        assert results['from_address'] == 'no-reply@mydomain.com'
        assert 'smtp_server_username' not in results
        assert 'smtp_server_password' not in results



# Copyright: (c) 2013, James Cammarata <jcammarata@ansible.com>
# Copyright: (c) 2018, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os.path
import re
import shutil
import sys
import time
import yaml

from jinja2 import Environment, FileSystemLoader

import ansible.constants as C
from ansible import context
from ansible.cli import CLI
from ansible.cli.arguments import optparse_helpers as opt_help
from ansible.errors import AnsibleError, AnsibleOptionsError
from ansible.galaxy import Galaxy
from ansible.galaxy.api import GalaxyAPI
from ansible.galaxy.login import GalaxyLogin
from ansible.galaxy.role import GalaxyRole
from ansible.galaxy.token import GalaxyToken
from ansible.module_utils._text import to_native, to_text
from ansible.playbook.role.requirement import RoleRequirement
from ansible.utils.display import Display

display = Display()


class GalaxyCLI(CLI):
    '''command to manage Ansible roles in shared repositories, the default of which is Ansible Galaxy *https://galaxy.ansible.com*.'''

    SKIP_INFO_KEYS = ("name", "description", "readme_html", "related", "summary_fields", "average_aw_composite", "average_aw_score", "url")
    VALID_ACTIONS = frozenset(("delete", "import", "info", "init", "install", "list", "login", "remove", "search", "setup"))

    def __init__(self, args):
        self.api = None
        self.galaxy = None
        super(GalaxyCLI, self).__init__(args)

    def set_action(self):

        super(GalaxyCLI, self).set_action()

        # specific to actions
        if self.action == "delete":
            self.parser.set_usage("usage: %prog delete [options] github_user github_repo")
            self.parser.set_description("Removes the role from Galaxy. It does not remove or alter the actual GitHub repository.")
        elif self.action == "import":
            self.parser.set_usage("usage: %prog import [options] github_user github_repo")
            self.parser.set_description("Import a role.")
            self.parser.add_option('--no-wait', dest='wait', action='store_false', default=True, help='Don\'t wait for import results.')
            self.parser.add_option('--branch', dest='reference',
                                   help='The name of a branch to import. Defaults to the repository\'s default branch (usually master)')
            self.parser.add_option('--role-name', dest='role_name', help='The name the role should have, if different than the repo name')
            self.parser.add_option('--status', dest='check_status', action='store_true', default=False,
                                   help='Check the status of the most recent import request for given github_user/github_repo.')
        elif self.action == "info":
            self.parser.set_usage("usage: %prog info [options] role_name[,version]")
            self.parser.set_description("View more details about a specific role.")
        elif self.action == "init":
            self.parser.set_usage("usage: %prog init [options] role_name")
            self.parser.set_description("Initialize new role with the base structure of a role.")
            self.parser.add_option('--init-path', dest='init_path', default="./",
                                   help='The path in which the skeleton role will be created. The default is the current working directory.')
            self.parser.add_option('--type', dest='role_type', action='store', default='default',
                                   help="Initialize using an alternate role type. Valid types include: 'container', 'apb' and 'network'.")
            self.parser.add_option('--role-skeleton', dest='role_skeleton', default=C.GALAXY_ROLE_SKELETON,
                                   help='The path to a role skeleton that the new role should be based upon.')
        elif self.action == "install":
            self.parser.set_usage("usage: %prog install [options] [-r FILE | role_name(s)[,version] | scm+role_repo_url[,version] | tar_file(s)]")
            self.parser.set_description("Install Roles from file(s), URL(s) or tar file(s)")
            self.parser.add_option('-i', '--ignore-errors', dest='ignore_errors', action='store_true', default=False,
                                   help='Ignore errors and continue with the next specified role.')
            self.parser.add_option('-n', '--no-deps', dest='no_deps', action='store_true', default=False, help='Don\'t download roles listed as dependencies')
            self.parser.add_option('-r', '--role-file', dest='role_file', help='A file containing a list of roles to be imported')
            self.parser.add_option('-g', '--keep-scm-meta', dest='keep_scm_meta', action='store_true',
                                   default=False, help='Use tar instead of the scm archive option when packaging the role')
        elif self.action == "remove":
            self.parser.set_usage("usage: %prog remove role1 role2 ...")
            self.parser.set_description("Delete a role from roles_path.")
        elif self.action == "list":
            self.parser.set_usage("usage: %prog list [role_name]")
            self.parser.set_description("Show the name and version of each role installed in the roles_path.")
        elif self.action == "login":
            self.parser.set_usage("usage: %prog login [options]")
            self.parser.set_description("Login to api.github.com server in order to use ansible-galaxy sub command such as 'import', 'delete' and 'setup'.")
            self.parser.add_option('--github-token', dest='token', default=None, help='Identify with github token rather than username and password.')
        elif self.action == "search":
            self.parser.set_usage("usage: %prog search [searchterm1 searchterm2] [--galaxy-tags galaxy_tag1,galaxy_tag2] [--platforms platform1,platform2] "
                                  "[--author username]")
            self.parser.add_option('--platforms', dest='platforms', help='list of OS platforms to filter by')
            self.parser.add_option('--galaxy-tags', dest='galaxy_tags', help='list of galaxy tags to filter by')
            self.parser.add_option('--author', dest='author', help='GitHub username')
            self.parser.set_description("Search the Galaxy database by tags, platforms, author and multiple keywords.")
        elif self.action == "setup":
            self.parser.set_usage("usage: %prog setup [options] source github_user github_repo secret")
            self.parser.add_option('--remove', dest='remove_id', default=None,
                                   help='Remove the integration matching the provided ID value. Use --list to see ID values.')
            self.parser.add_option('--list', dest="setup_list", action='store_true', default=False, help='List all of your integrations.')
            self.parser.set_description("Manage the integration between Galaxy and the given source.")
        # options that apply to more than one action
        if self.action in ['init', 'info']:
            self.parser.add_option('--offline', dest='offline', default=False, action='store_true', help="Don't query the galaxy API when creating roles")

        if self.action not in ("delete", "import", "init", "login", "setup"):
            # NOTE: while the option type=str, the default is a list, and the
            # callback will set the value to a list.
            self.parser.add_option('-p', '--roles-path', dest='roles_path', action="callback", callback=opt_help.unfrack_paths, default=C.DEFAULT_ROLES_PATH,
                                   help='The path to the directory containing your roles. The default is the roles_path configured in your ansible.cfg'
                                        ' file (/etc/ansible/roles if not configured)', type='str')
        if self.action in ("init", "install"):
            self.parser.add_option('-f', '--force', dest='force', action='store_true', default=False, help='Force overwriting an existing role')
            if self.action == "install":
                self.parser.add_option('--force-with-deps', dest='force_with_deps', action='store_true', default=False,
                                       help="Force overwriting an existing role and it's dependencies")

    def init_parser(self):
        ''' create an options parser for bin/ansible '''

        super(GalaxyCLI, self).init_parser(
            usage="usage: %%prog [%s] [--help] [options] ..." % "|".join(sorted(self.VALID_ACTIONS)),
            epilog="\nSee '%s <command> --help' for more information on a specific command.\n\n" % os.path.basename(sys.argv[0]),
            desc="Perform various Role related operations.",
        )

        # common
        self.parser.add_option('-s', '--server', dest='api_server', default=C.GALAXY_SERVER, help='The API server destination')
        self.parser.add_option('-c', '--ignore-certs', action='store_true', dest='ignore_certs', default=C.GALAXY_IGNORE_CERTS,
                               help='Ignore SSL certificate validation errors.')
        self.set_action()

    def post_process_args(self, options, args):
        options, args = super(GalaxyCLI, self).post_process_args(options, args)
        display.verbosity = options.verbosity
        return options, args

    def run(self):

        super(GalaxyCLI, self).run()

        self.galaxy = Galaxy()

        self.api = GalaxyAPI(self.galaxy)
        self.execute()

    @staticmethod
    def exit_without_ignore(rc=1):
        """
        Exits with the specified return code unless the
        option --ignore-errors was specified
        """
        if not context.CLIARGS['ignore_errors']:
            raise AnsibleError('- you can use --ignore-errors to skip failed roles and finish processing the list.')

    @staticmethod
    def _display_role_info(role_info):

        text = [u"", u"Role: %s" % to_text(role_info['name'])]
        text.append(u"\tdescription: %s" % role_info.get('description', ''))

        for k in sorted(role_info.keys()):

            if k in GalaxyCLI.SKIP_INFO_KEYS:
                continue

            if isinstance(role_info[k], dict):
                text.append(u"\t%s:" % (k))
                for key in sorted(role_info[k].keys()):
                    if key in GalaxyCLI.SKIP_INFO_KEYS:
                        continue
                    text.append(u"\t\t%s: %s" % (key, role_info[k][key]))
            else:
                text.append(u"\t%s: %s" % (k, role_info[k]))

        return u'\n'.join(text)

############################
# execute actions
############################

    def execute_init(self):
        """
        creates the skeleton framework of a role that complies with the galaxy metadata format.
        """

        init_path = context.CLIARGS['init_path']
        force = context.CLIARGS['force']
        role_skeleton = context.CLIARGS['role_skeleton']

        role_name = context.CLIARGS['args'][0].strip() if context.CLIARGS['args'] else None
        if not role_name:
            raise AnsibleOptionsError("- no role name specified for init")
        role_path = os.path.join(init_path, role_name)
        if os.path.exists(role_path):
            if os.path.isfile(role_path):
                raise AnsibleError("- the path %s already exists, but is a file - aborting" % role_path)
            elif not force:
                raise AnsibleError("- the directory %s already exists."
                                   "you can use --force to re-initialize this directory,\n"
                                   "however it will reset any main.yml files that may have\n"
                                   "been modified there already." % role_path)

        inject_data = dict(
            role_name=role_name,
            author='your name',
            description='your description',
            company='your company (optional)',
            license='license (GPL-2.0-or-later, MIT, etc)',
            issue_tracker_url='http://example.com/issue/tracker',
            min_ansible_version='2.4',
            role_type=context.CLIARGS['role_type']
        )

        # create role directory
        if not os.path.exists(role_path):
            os.makedirs(role_path)

        if role_skeleton is not None:
            skeleton_ignore_expressions = C.GALAXY_ROLE_SKELETON_IGNORE
        else:
            role_skeleton = self.galaxy.default_role_skeleton_path
            skeleton_ignore_expressions = ['^.*/.git_keep$']

        role_skeleton = os.path.expanduser(role_skeleton)
        skeleton_ignore_re = [re.compile(x) for x in skeleton_ignore_expressions]

        template_env = Environment(loader=FileSystemLoader(role_skeleton))

        for root, dirs, files in os.walk(role_skeleton, topdown=True):
            rel_root = os.path.relpath(root, role_skeleton)
            in_templates_dir = rel_root.split(os.sep, 1)[0] == 'templates'
            dirs[:] = [d for d in dirs if not any(r.match(d) for r in skeleton_ignore_re)]

            for f in files:
                filename, ext = os.path.splitext(f)
                if any(r.match(os.path.join(rel_root, f)) for r in skeleton_ignore_re):
                    continue
                elif ext == ".j2" and not in_templates_dir:
                    src_template = os.path.join(rel_root, f)
                    dest_file = os.path.join(role_path, rel_root, filename)
                    template_env.get_template(src_template).stream(inject_data).dump(dest_file)
                else:
                    f_rel_path = os.path.relpath(os.path.join(root, f), role_skeleton)
                    shutil.copyfile(os.path.join(root, f), os.path.join(role_path, f_rel_path))

            for d in dirs:
                dir_path = os.path.join(role_path, rel_root, d)
                if not os.path.exists(dir_path):
                    os.makedirs(dir_path)

        display.display("- %s was created successfully" % role_name)

    def execute_info(self):
        """
        prints out detailed information about an installed role as well as info available from the galaxy API.
        """

        if not context.CLIARGS['args']:
            # the user needs to specify a role
            raise AnsibleOptionsError("- you must specify a user/role name")

        roles_path = context.CLIARGS['roles_path']

        data = ''
        for role in context.CLIARGS['args']:

            role_info = {'path': roles_path}
            gr = GalaxyRole(self.galaxy, role)

            install_info = gr.install_info
            if install_info:
                if 'version' in install_info:
                    install_info['installed_version'] = install_info['version']
                    del install_info['version']
                role_info.update(install_info)

            remote_data = False
            if not context.CLIARGS['offline']:
                remote_data = self.api.lookup_role_by_name(role, False)

            if remote_data:
                role_info.update(remote_data)

            if gr.metadata:
                role_info.update(gr.metadata)

            req = RoleRequirement()
            role_spec = req.role_yaml_parse({'role': role})
            if role_spec:
                role_info.update(role_spec)

            data = self._display_role_info(role_info)
            # FIXME: This is broken in both 1.9 and 2.0 as
            # _display_role_info() always returns something
            if not data:
                data = u"\n- the role %s was not found" % role

        self.pager(data)

    def execute_install(self):
        """
        uses the args list of roles to be installed, unless -f was specified. The list of roles
        can be a name (which will be downloaded via the galaxy API and github), or it can be a local .tar.gz file.
        """
        role_file = context.CLIARGS['role_file']

        if not context.CLIARGS['args'] and role_file is None:
            # the user needs to specify one of either --role-file or specify a single user/role name
            raise AnsibleOptionsError("- you must specify a user/role name or a roles file")

        no_deps = context.CLIARGS['no_deps']
        force_deps = context.CLIARGS['force_with_deps']

        if no_deps and force_deps:
            raise AnsibleOptionsError("You cannot both force dependencies and no dependencies")

        force = context.CLIARGS['force'] or force_deps

        roles_left = []
        if role_file:
            try:
                f = open(role_file, 'r')
                if role_file.endswith('.yaml') or role_file.endswith('.yml'):
                    try:
                        required_roles = yaml.safe_load(f.read())
                    except Exception as e:
                        raise AnsibleError("Unable to load data from the requirements file (%s): %s" % (role_file, to_native(e)))

                    if required_roles is None:
                        raise AnsibleError("No roles found in file: %s" % role_file)

                    for role in required_roles:
                        if "include" not in role:
                            role = RoleRequirement.role_yaml_parse(role)
                            display.vvv("found role %s in yaml file" % str(role))
                            if "name" not in role and "scm" not in role:
                                raise AnsibleError("Must specify name or src for role")
                            roles_left.append(GalaxyRole(self.galaxy, **role))
                        else:
                            with open(role["include"]) as f_include:
                                try:
                                    roles_left += [
                                        GalaxyRole(self.galaxy, **r) for r in
                                        (RoleRequirement.role_yaml_parse(i) for i in yaml.safe_load(f_include))
                                    ]
                                except Exception as e:
                                    msg = "Unable to load data from the include requirements file: %s %s"
                                    raise AnsibleError(msg % (role_file, e))
                else:
                    raise AnsibleError("Invalid role requirements file")
                f.close()
            except (IOError, OSError) as e:
                raise AnsibleError('Unable to open %s: %s' % (role_file, to_native(e)))
        else:
            # roles were specified directly, so we'll just go out grab them
            # (and their dependencies, unless the user doesn't want us to).
            for rname in context.CLIARGS['args']:
                role = RoleRequirement.role_yaml_parse(rname.strip())
                roles_left.append(GalaxyRole(self.galaxy, **role))

        for role in roles_left:
            # only process roles in roles files when names matches if given
            if role_file and context.CLIARGS['args'] and role.name not in context.CLIARGS['args']:
                display.vvv('Skipping role %s' % role.name)
                continue

            display.vvv('Processing role %s ' % role.name)

            # query the galaxy API for the role data

            if role.install_info is not None:
                if role.install_info['version'] != role.version or force:
                    if force:
                        display.display('- changing role %s from %s to %s' %
                                        (role.name, role.install_info['version'], role.version or "unspecified"))
                        role.remove()
                    else:
                        display.warning('- %s (%s) is already installed - use --force to change version to %s' %
                                        (role.name, role.install_info['version'], role.version or "unspecified"))
                        continue
                else:
                    if not force:
                        display.display('- %s is already installed, skipping.' % str(role))
                        continue

            try:
                installed = role.install()
            except AnsibleError as e:
                display.warning(u"- %s was NOT installed successfully: %s " % (role.name, to_text(e)))
                self.exit_without_ignore()
                continue

            # install dependencies, if we want them
            if not no_deps and installed:
                if not role.metadata:
                    display.warning("Meta file %s is empty. Skipping dependencies." % role.path)
                else:
                    role_dependencies = role.metadata.get('dependencies') or []
                    for dep in role_dependencies:
                        display.debug('Installing dep %s' % dep)
                        dep_req = RoleRequirement()
                        dep_info = dep_req.role_yaml_parse(dep)
                        dep_role = GalaxyRole(self.galaxy, **dep_info)
                        if '.' not in dep_role.name and '.' not in dep_role.src and dep_role.scm is None:
                            # we know we can skip this, as it's not going to
                            # be found on galaxy.ansible.com
                            continue
                        if dep_role.install_info is None:
                            if dep_role not in roles_left:
                                display.display('- adding dependency: %s' % to_text(dep_role))
                                roles_left.append(dep_role)
                            else:
                                display.display('- dependency %s already pending installation.' % dep_role.name)
                        else:
                            if dep_role.install_info['version'] != dep_role.version:
                                if force_deps:
                                    display.display('- changing dependant role %s from %s to %s' %
                                                    (dep_role.name, dep_role.install_info['version'], dep_role.version or "unspecified"))
                                    dep_role.remove()
                                    roles_left.append(dep_role)
                                else:
                                    display.warning('- dependency %s from role %s differs from already installed version (%s), skipping' %
                                                    (to_text(dep_role), role.name, dep_role.install_info['version']))
                            else:
                                if force_deps:
                                    roles_left.append(dep_role)
                                else:
                                    display.display('- dependency %s is already installed, skipping.' % dep_role.name)

            if not installed:
                display.warning("- %s was NOT installed successfully." % role.name)
                self.exit_without_ignore()

        return 0

    def execute_remove(self):
        """
        removes the list of roles passed as arguments from the local system.
        """

        if not context.CLIARGS['args']:
            raise AnsibleOptionsError('- you must specify at least one role to remove.')

        for role_name in context.CLIARGS['args']:
            role = GalaxyRole(self.galaxy, role_name)
            try:
                if role.remove():
                    display.display('- successfully removed %s' % role_name)
                else:
                    display.display('- %s is not installed, skipping.' % role_name)
            except Exception as e:
                raise AnsibleError("Failed to remove role %s: %s" % (role_name, to_native(e)))

        return 0

    def execute_list(self):
        """
        lists the roles installed on the local system or matches a single role passed as an argument.
        """

        if len(context.CLIARGS['args']) > 1:
            raise AnsibleOptionsError("- please specify only one role to list, or specify no roles to see a full list")

        def _display_role(gr):
            install_info = gr.install_info
            version = None
            if install_info:
                version = install_info.get("version", None)
            if not version:
                version = "(unknown version)"
            display.display("- %s, %s" % (gr.name, version))

        if context.CLIARGS['args']:
            # show the requested role, if it exists
            name = context.CLIARGS['args'][0]
            gr = GalaxyRole(self.galaxy, name)
            if gr.metadata:
                display.display('# %s' % os.path.dirname(gr.path))
                _display_role(gr)
            else:
                display.display("- the role %s was not found" % name)
        else:
            # show all valid roles in the roles_path directory
            roles_path = context.CLIARGS['roles_path']
            path_found = False
            warnings = []
            for path in roles_path:
                role_path = os.path.expanduser(path)
                if not os.path.exists(role_path):
                    warnings.append("- the configured path %s does not exist." % role_path)
                    continue
                elif not os.path.isdir(role_path):
                    warnings.append("- the configured path %s, exists, but it is not a directory." % role_path)
                    continue
                display.display('# %s' % role_path)
                path_files = os.listdir(role_path)
                path_found = True
                for path_file in path_files:
                    gr = GalaxyRole(self.galaxy, path_file, path=path)
                    if gr.metadata:
                        _display_role(gr)
            for w in warnings:
                display.warning(w)
            if not path_found:
                raise AnsibleOptionsError("- None of the provided paths was usable. Please specify a valid path with --roles-path")
        return 0

    def execute_search(self):
        ''' searches for roles on the Ansible Galaxy server'''
        page_size = 1000
        search = None

        if context.CLIARGS['args']:
            search = '+'.join(context.CLIARGS['args'])

        if not search and not context.CLIARGS['platforms'] and not context.CLIARGS['galaxy_tags'] and not context.CLIARGS['author']:
            raise AnsibleError("Invalid query. At least one search term, platform, galaxy tag or author must be provided.")

        response = self.api.search_roles(search, platforms=context.CLIARGS['platforms'],
                                         tags=context.CLIARGS['galaxy_tags'], author=context.CLIARGS['author'], page_size=page_size)

        if response['count'] == 0:
            display.display("No roles match your search.", color=C.COLOR_ERROR)
            return True

        data = [u'']

        if response['count'] > page_size:
            data.append(u"Found %d roles matching your search. Showing first %s." % (response['count'], page_size))
        else:
            data.append(u"Found %d roles matching your search:" % response['count'])

        max_len = []
        for role in response['results']:
            max_len.append(len(role['username'] + '.' + role['name']))
        name_len = max(max_len)
        format_str = u" %%-%ds %%s" % name_len
        data.append(u'')
        data.append(format_str % (u"Name", u"Description"))
        data.append(format_str % (u"----", u"-----------"))
        for role in response['results']:
            data.append(format_str % (u'%s.%s' % (role['username'], role['name']), role['description']))

        data = u'\n'.join(data)
        self.pager(data)

        return True

    def execute_login(self):
        """
        verify user's identify via GitHub and retrieve an auth token from Ansible Galaxy.
        """
        # Authenticate with github and retrieve a token
        if context.CLIARGS['token'] is None:
            if C.GALAXY_TOKEN:
                github_token = C.GALAXY_TOKEN
            else:
                login = GalaxyLogin(self.galaxy)
                github_token = login.create_github_token()
        else:
            github_token = context.CLIARGS['token']

        galaxy_response = self.api.authenticate(github_token)

        if context.CLIARGS['token'] is None and C.GALAXY_TOKEN is None:
            # Remove the token we created
            login.remove_github_token()

        # Store the Galaxy token
        token = GalaxyToken()
        token.set(galaxy_response['token'])

        display.display("Successfully logged into Galaxy as %s" % galaxy_response['username'])
        return 0

    def execute_import(self):
        """ used to import a role into Ansible Galaxy """

        colors = {
            'INFO': 'normal',
            'WARNING': C.COLOR_WARN,
            'ERROR': C.COLOR_ERROR,
            'SUCCESS': C.COLOR_OK,
            'FAILED': C.COLOR_ERROR,
        }

        if len(context.CLIARGS['args']) < 2:
            raise AnsibleError("Expected a github_username and github_repository. Use --help.")

        github_user = to_text(context.CLIARGS['args'][0], errors='surrogate_or_strict')
        github_repo = to_text(context.CLIARGS['args'][1], errors='surrogate_or_strict')

        if context.CLIARGS['check_status']:
            task = self.api.get_import_task(github_user=github_user, github_repo=github_repo)
        else:
            # Submit an import request
            task = self.api.create_import_task(github_user, github_repo,
                                               reference=context.CLIARGS['reference'],
                                               role_name=context.CLIARGS['role_name'])

            if len(task) > 1:
                # found multiple roles associated with github_user/github_repo
                display.display("WARNING: More than one Galaxy role associated with GitHub repo %s/%s." % (github_user, github_repo),
                                color='yellow')
                display.display("The following Galaxy roles are being updated:" + u'\n', color=C.COLOR_CHANGED)
                for t in task:
                    display.display('%s.%s' % (t['summary_fields']['role']['namespace'], t['summary_fields']['role']['name']), color=C.COLOR_CHANGED)
                display.display(u'\nTo properly namespace this role, remove each of the above and re-import %s/%s from scratch' % (github_user, github_repo),
                                color=C.COLOR_CHANGED)
                return 0
            # found a single role as expected
            display.display("Successfully submitted import request %d" % task[0]['id'])
            if not context.CLIARGS['wait']:
                display.display("Role name: %s" % task[0]['summary_fields']['role']['name'])
                display.display("Repo: %s/%s" % (task[0]['github_user'], task[0]['github_repo']))

        if context.CLIARGS['check_status'] or context.CLIARGS['wait']:
            # Get the status of the import
            msg_list = []
            finished = False
            while not finished:
                task = self.api.get_import_task(task_id=task[0]['id'])
                for msg in task[0]['summary_fields']['task_messages']:
                    if msg['id'] not in msg_list:
                        display.display(msg['message_text'], color=colors[msg['message_type']])
                        msg_list.append(msg['id'])
                if task[0]['state'] in ['SUCCESS', 'FAILED']:
                    finished = True
                else:
                    time.sleep(10)

        return 0

    def execute_setup(self):
        """ Setup an integration from GitHub or Travis for Ansible Galaxy roles"""

        if context.CLIARGS['setup_list']:
            # List existing integration secrets
            secrets = self.api.list_secrets()
            if len(secrets) == 0:
                # None found
                display.display("No integrations found.")
                return 0
            display.display(u'\n' + "ID         Source     Repo", color=C.COLOR_OK)
            display.display("---------- ---------- ----------", color=C.COLOR_OK)
            for secret in secrets:
                display.display("%-10s %-10s %s/%s" % (secret['id'], secret['source'], secret['github_user'],
                                                       secret['github_repo']), color=C.COLOR_OK)
            return 0

        if context.CLIARGS['remove_id']:
            # Remove a secret
            self.api.remove_secret(context.CLIARGS['remove_id'])
            display.display("Secret removed. Integrations using this secret will not longer work.", color=C.COLOR_OK)
            return 0

        if len(context.CLIARGS['args']) < 4:
            raise AnsibleError("Missing one or more arguments. Expecting: source github_user github_repo secret")

        source = context.CLIARGS['args'][0]
        github_user = context.CLIARGS['args'][1]
        github_repo = context.CLIARGS['args'][2]
        secret = context.CLIARGS['args'][3]

        resp = self.api.add_secret(source, github_user, github_repo, secret)
        display.display("Added integration for %s %s/%s" % (resp['source'], resp['github_user'], resp['github_repo']))

        return 0

    def execute_delete(self):
        """ Delete a role from Ansible Galaxy. """

        if len(context.CLIARGS['args']) < 2:
            raise AnsibleError("Missing one or more arguments. Expected: github_user github_repo")

        github_user = context.CLIARGS['args'][0]
        github_repo = context.CLIARGS['args'][1]
        resp = self.api.delete_role(github_user, github_repo)

        if len(resp['deleted_roles']) > 1:
            display.display("Deleted the following roles:")
            display.display("ID     User            Name")
            display.display("------ --------------- ----------")
            for role in resp['deleted_roles']:
                display.display("%-8s %-15s %s" % (role.id, role.namespace, role.name))

        display.display(resp['status'])

        return True



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_endpoint_control_forticlient_registration_sync
short_description: Configure FortiClient registration synchronization settings in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure endpoint_control feature and forticlient_registration_sync category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    endpoint_control_forticlient_registration_sync:
        description:
            - Configure FortiClient registration synchronization settings.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            peer-ip:
                description:
                    - IP address of the peer FortiGate for endpoint license synchronization.
            peer-name:
                description:
                    - Peer name.
                required: true
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure FortiClient registration synchronization settings.
    fortios_endpoint_control_forticlient_registration_sync:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      endpoint_control_forticlient_registration_sync:
        state: "present"
        peer-ip: "<your_own_value>"
        peer-name: "<your_own_value>"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_endpoint_control_forticlient_registration_sync_data(json):
    option_list = ['peer-ip', 'peer-name']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def endpoint_control_forticlient_registration_sync(data, fos):
    vdom = data['vdom']
    endpoint_control_forticlient_registration_sync_data = data['endpoint_control_forticlient_registration_sync']
    filtered_data = filter_endpoint_control_forticlient_registration_sync_data(endpoint_control_forticlient_registration_sync_data)
    if endpoint_control_forticlient_registration_sync_data['state'] == "present":
        return fos.set('endpoint-control',
                       'forticlient-registration-sync',
                       data=filtered_data,
                       vdom=vdom)

    elif endpoint_control_forticlient_registration_sync_data['state'] == "absent":
        return fos.delete('endpoint-control',
                          'forticlient-registration-sync',
                          mkey=filtered_data['peer-name'],
                          vdom=vdom)


def fortios_endpoint_control(data, fos):
    login(data)

    methodlist = ['endpoint_control_forticlient_registration_sync']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "endpoint_control_forticlient_registration_sync": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "peer-ip": {"required": False, "type": "str"},
                "peer-name": {"required": True, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_endpoint_control(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit tests for Ansible module: na_ontap_job_schedule '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_job_schedule \
    import NetAppONTAPJob as job_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, data=None):
        ''' save arguments '''
        self.kind = kind
        self.params = data
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.kind == 'job':
            xml = self.build_job_schedule_cron_info(self.params)
        elif self.kind == 'job_multiple':
            xml = self.build_job_schedule_multiple_cron_info(self.params)
        # TODO: mock invoke_elem for autosupport calls
        elif self.kind == 'vserver':
            xml = self.build_vserver_info()
        self.xml_out = xml
        return xml

    def autosupport_log(self):
        ''' Mock autosupport log method, returns None '''
        return None

    @staticmethod
    def build_job_schedule_cron_info(job_details):
        ''' build xml data for vserser-info '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'num-records': 1,
            'attributes-list': {
                'job-schedule-cron-info': {
                    'job-schedule-name': job_details['name'],
                    'job-schedule-cron-minute': {
                        'cron-minute': job_details['minutes']
                    }
                }
            }
        }
        xml.translate_struct(attributes)
        return xml

    @staticmethod
    def build_job_schedule_multiple_cron_info(job_details):
        ''' build xml data for vserser-info '''
        print("CALLED MULTIPLE BUILD")
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'num-records': 1,
            'attributes-list': {
                'job-schedule-cron-info': {
                    'job-schedule-name': job_details['name'],
                    'job-schedule-cron-minute': [
                        {'cron-minute': '25'},
                        {'cron-minute': '35'}
                    ],
                    'job-schedule-cron-month': [
                        {'cron-month': '5'},
                        {'cron-month': '10'}
                    ]
                }
            }
        }
        xml.translate_struct(attributes)
        return xml


class TestMyModule(unittest.TestCase):
    ''' Unit tests for na_ontap_job_schedule '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.mock_job = {
            'name': 'test_job',
            'minutes': '25'
        }

    def mock_args(self):
        return {
            'name': self.mock_job['name'],
            'job_minutes': [self.mock_job['minutes']],
            'hostname': 'test',
            'username': 'test_user',
            'password': 'test_pass!'
        }

    def get_job_mock_object(self, kind=None):
        """
        Helper method to return an na_ontap_job_schedule object
        :param kind: passes this param to MockONTAPConnection()
        :return: na_ontap_job_schedule object
        """
        job_obj = job_module()
        job_obj.autosupport_log = Mock(return_value=None)
        if kind is None:
            job_obj.server = MockONTAPConnection()
        else:
            job_obj.server = MockONTAPConnection(kind=kind, data=self.mock_job)
        return job_obj

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            job_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    def test_get_nonexistent_job(self):
        ''' Test if get_job_schedule returns None for non-existent job '''
        set_module_args(self.mock_args())
        result = self.get_job_mock_object().get_job_schedule()
        assert result is None

    def test_get_existing_job(self):
        ''' Test if get_job_schedule retuns job details for existing job '''
        data = self.mock_args()
        set_module_args(data)
        result = self.get_job_mock_object('job').get_job_schedule()
        assert result['name'] == self.mock_job['name']
        assert result['job_minutes'] == data['job_minutes']

    def test_get_existing_job_multiple_minutes(self):
        ''' Test if get_job_schedule retuns job details for existing job '''
        set_module_args(self.mock_args())
        result = self.get_job_mock_object('job_multiple').get_job_schedule()
        print(str(result))
        assert result['name'] == self.mock_job['name']
        assert result['job_minutes'] == ['25', '35']
        assert result['job_months'] == ['5', '10']

    def test_create_error_missing_param(self):
        ''' Test if create throws an error if job_minutes is not specified'''
        data = self.mock_args()
        del data['job_minutes']
        set_module_args(data)
        with pytest.raises(AnsibleFailJson) as exc:
            self.get_job_mock_object('job').create_job_schedule()
        msg = 'Error: missing required parameter job_minutes for create'
        assert exc.value.args[0]['msg'] == msg

    def test_successful_create(self):
        ''' Test successful create '''
        set_module_args(self.mock_args())
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_job_mock_object().apply()
        assert exc.value.args[0]['changed']

    def test_create_idempotency(self):
        ''' Test create idempotency '''
        set_module_args(self.mock_args())
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_job_mock_object('job').apply()
        assert not exc.value.args[0]['changed']

    def test_successful_delete(self):
        ''' Test delete existing job '''
        data = self.mock_args()
        data['state'] = 'absent'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_job_mock_object('job').apply()
        assert exc.value.args[0]['changed']

    def test_delete_idempotency(self):
        ''' Test delete idempotency '''
        data = self.mock_args()
        data['state'] = 'absent'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_job_mock_object().apply()
        assert not exc.value.args[0]['changed']

    def test_successful_modify(self):
        ''' Test successful modify job_minutes '''
        data = self.mock_args()
        data['job_minutes'] = ['20']
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_job_mock_object('job').apply()
        assert exc.value.args[0]['changed']

    def test_modify_idempotency(self):
        ''' Test modify idempotency '''
        data = self.mock_args()
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_job_mock_object('job').apply()
        assert not exc.value.args[0]['changed']



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_vpn_ipsec_manualkey_interface
short_description: Configure IPsec manual keys in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify vpn_ipsec feature and manualkey_interface category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    vpn_ipsec_manualkey_interface:
        description:
            - Configure IPsec manual keys.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            addr-type:
                description:
                    - IP version to use for IP packets.
                choices:
                    - 4
                    - 6
            auth-alg:
                description:
                    - Authentication algorithm. Must be the same for both ends of the tunnel.
                choices:
                    - null
                    - md5
                    - sha1
                    - sha256
                    - sha384
                    - sha512
            auth-key:
                description:
                    - Hexadecimal authentication key in 16-digit (8-byte) segments separated by hyphens.
            enc-alg:
                description:
                    - Encryption algorithm. Must be the same for both ends of the tunnel.
                choices:
                    - null
                    - des
            enc-key:
                description:
                    - Hexadecimal encryption key in 16-digit (8-byte) segments separated by hyphens.
            interface:
                description:
                    - Name of the physical, aggregate, or VLAN interface. Source system.interface.name.
            ip-version:
                description:
                    - IP version to use for VPN interface.
                choices:
                    - 4
                    - 6
            local-gw:
                description:
                    - IPv4 address of the local gateway's external interface.
            local-gw6:
                description:
                    - Local IPv6 address of VPN gateway.
            local-spi:
                description:
                    - Local SPI, a hexadecimal 8-digit (4-byte) tag. Discerns between two traffic streams with different encryption rules.
            name:
                description:
                    - IPsec tunnel name.
                required: true
            npu-offload:
                description:
                    - Enable/disable offloading IPsec VPN manual key sessions to NPUs.
                choices:
                    - enable
                    - disable
            remote-gw:
                description:
                    - IPv4 address of the remote gateway's external interface.
            remote-gw6:
                description:
                    - Remote IPv6 address of VPN gateway.
            remote-spi:
                description:
                    - Remote SPI, a hexadecimal 8-digit (4-byte) tag. Discerns between two traffic streams with different encryption rules.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure IPsec manual keys.
    fortios_vpn_ipsec_manualkey_interface:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      vpn_ipsec_manualkey_interface:
        state: "present"
        addr-type: "4"
        auth-alg: "null"
        auth-key: "<your_own_value>"
        enc-alg: "null"
        enc-key: "<your_own_value>"
        interface: "<your_own_value> (source system.interface.name)"
        ip-version: "4"
        local-gw: "<your_own_value>"
        local-gw6: "<your_own_value>"
        local-spi: "<your_own_value>"
        name: "default_name_13"
        npu-offload: "enable"
        remote-gw: "<your_own_value>"
        remote-gw6: "<your_own_value>"
        remote-spi: "<your_own_value>"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_vpn_ipsec_manualkey_interface_data(json):
    option_list = ['addr-type', 'auth-alg', 'auth-key',
                   'enc-alg', 'enc-key', 'interface',
                   'ip-version', 'local-gw', 'local-gw6',
                   'local-spi', 'name', 'npu-offload',
                   'remote-gw', 'remote-gw6', 'remote-spi']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def vpn_ipsec_manualkey_interface(data, fos):
    vdom = data['vdom']
    vpn_ipsec_manualkey_interface_data = data['vpn_ipsec_manualkey_interface']
    flattened_data = flatten_multilists_attributes(vpn_ipsec_manualkey_interface_data)
    filtered_data = filter_vpn_ipsec_manualkey_interface_data(flattened_data)
    if vpn_ipsec_manualkey_interface_data['state'] == "present":
        return fos.set('vpn.ipsec',
                       'manualkey-interface',
                       data=filtered_data,
                       vdom=vdom)

    elif vpn_ipsec_manualkey_interface_data['state'] == "absent":
        return fos.delete('vpn.ipsec',
                          'manualkey-interface',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_vpn_ipsec(data, fos):
    login(data)

    if data['vpn_ipsec_manualkey_interface']:
        resp = vpn_ipsec_manualkey_interface(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "vpn_ipsec_manualkey_interface": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "addr-type": {"required": False, "type": "str",
                              "choices": ["4", "6"]},
                "auth-alg": {"required": False, "type": "str",
                             "choices": ["null", "md5", "sha1",
                                         "sha256", "sha384", "sha512"]},
                "auth-key": {"required": False, "type": "str"},
                "enc-alg": {"required": False, "type": "str",
                            "choices": ["null", "des"]},
                "enc-key": {"required": False, "type": "str"},
                "interface": {"required": False, "type": "str"},
                "ip-version": {"required": False, "type": "str",
                               "choices": ["4", "6"]},
                "local-gw": {"required": False, "type": "str"},
                "local-gw6": {"required": False, "type": "str"},
                "local-spi": {"required": False, "type": "str"},
                "name": {"required": True, "type": "str"},
                "npu-offload": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "remote-gw": {"required": False, "type": "str"},
                "remote-gw6": {"required": False, "type": "str"},
                "remote-spi": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_vpn_ipsec(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import re
import sys


def main():
    skip = set([
        'test/runner/requirements/constraints.txt',
        'test/runner/requirements/integration.cloud.azure.txt',
    ])

    for path in sys.argv[1:] or sys.stdin.read().splitlines():
        if path in skip:
            continue

        with open(path, 'r') as path_fd:
            for line, text in enumerate(path_fd.readlines()):
                match = re.search(r'^[^;#]*?([<>=])(?!.*sanity_ok.*)', text)

                if match:
                    print('%s:%d:%d: put constraints in `test/runner/requirements/constraints.txt`' % (
                        path, line + 1, match.start(1) + 1))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) 2018, Ansible Project
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


import fnmatch
import json
import os
import os.path
import re
import sys
from distutils.version import LooseVersion

import packaging.specifiers

from ansible.module_utils.urls import open_url


BUNDLED_RE = re.compile(b'\\b_BUNDLED_METADATA\\b')


def get_bundled_libs(paths):
    bundled_libs = set()
    for filename in fnmatch.filter(paths, 'lib/ansible/compat/*/__init__.py'):
        bundled_libs.add(filename)

    bundled_libs.add('lib/ansible/module_utils/distro/__init__.py')
    bundled_libs.add('lib/ansible/module_utils/six/__init__.py')
    bundled_libs.add('lib/ansible/module_utils/compat/ipaddress.py')
    # backports.ssl_match_hostname should be moved to its own file in the future
    bundled_libs.add('lib/ansible/module_utils/urls.py')

    return bundled_libs


def get_files_with_bundled_metadata(paths):
    with_metadata = set()
    for path in paths:
        if path == 'test/sanity/code-smell/update-bundled.py':
            continue

        with open(path, 'rb') as f:
            body = f.read()

        if BUNDLED_RE.search(body):
            with_metadata.add(path)

    return with_metadata


def get_bundled_metadata(filename):
    with open(filename, 'r') as module:
        for line in module:
            if line.strip().startswith('_BUNDLED_METADATA'):
                data = line[line.index('{'):].strip()
                break
        else:
            raise ValueError('Unable to check bundled library for update.  Please add'
                             ' _BUNDLED_METADATA dictionary to the library file with'
                             ' information on pypi name and bundled version.')
        metadata = json.loads(data)
    return metadata


def get_latest_applicable_version(pypi_data, constraints=None):
    latest_version = "0"
    if 'version_constraints' in metadata:
        version_specification = packaging.specifiers.SpecifierSet(metadata['version_constraints'])
        for version in pypi_data['releases']:
            if version in version_specification:
                if LooseVersion(version) > LooseVersion(latest_version):
                    latest_version = version
    else:
        latest_version = pypi_data['info']['version']

    return latest_version


if __name__ == '__main__':
    paths = sys.argv[1:] or sys.stdin.read().splitlines()

    bundled_libs = get_bundled_libs(paths)
    files_with_bundled_metadata = get_files_with_bundled_metadata(paths)

    for filename in files_with_bundled_metadata.difference(bundled_libs):
        print('{0}: ERROR: File contains _BUNDLED_METADATA but needs to be added to'
              ' test/sanity/code-smell/update-bundled.py'.format(filename))

    for filename in bundled_libs:
        try:
            metadata = get_bundled_metadata(filename)
        except ValueError as e:
            print('{0}: ERROR: {1}'.format(filename, e))
            continue
        except (IOError, OSError) as e:
            if e.errno == 2:
                print('{0}: ERROR: {1}.  Perhaps the bundled library has been removed'
                      ' or moved and the bundled library test needs to be modified as'
                      ' well?'.format(filename, e))

        pypi_fh = open_url('https://pypi.org/pypi/{0}/json'.format(metadata['pypi_name']))
        pypi_data = json.loads(pypi_fh.read().decode('utf-8'))

        constraints = metadata.get('version_constraints', None)
        latest_version = get_latest_applicable_version(pypi_data, constraints)

        if LooseVersion(metadata['version']) < LooseVersion(latest_version):
            print('{0}: UPDATE {1} from {2} to {3} {4}'.format(
                filename,
                metadata['pypi_name'],
                metadata['version'],
                latest_version,
                'https://pypi.org/pypi/{0}/json'.format(metadata['pypi_name'])))



#!/usr/bin/env python

import json
import sys

import ansible_collections.testns.testcoll.plugins.module_utils.leaf


def main():
    mu_result = ansible_collections.testns.testcoll.plugins.module_utils.leaf.thingtocall()
    print(json.dumps(dict(changed=False, source='user', mu_result=mu_result)))

    sys.exit()


if __name__ == '__main__':
    main()
